%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Power_Consumption_Calculation_Script.m - modified plotGPIOVoltCur3c6.m new oscilloscope
% format of time, Ch1(5v curr sensor), Ch2(nRst), Ch3(GPIO_0)
% JQ 100817
% % JQ 051818 need to add the 0current, scalecurrent, metercurrent as a curr
% % probe cal method channel 1 only
%   
% 
%   Follow instructions in TIDEP-0091 Design Guide in order to calculate 
%   power consumption
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_5vcur_nRST_GPIO0( fpath, fnameGIO, ofname, ofexcelname, plotsets, pretrigger, triggertime)
   % script to plot, output subset of oscilloscope plot

   testindex = 1; %es3 save&res pcal&meas 2 chirp PMICJ new board with PFM for PMIC

   meas_set = 3;
 
   if(testindex == 1)
      fpath = 'C:\ti'; %Change this to path containing Oscilloscope .csv file
      
      %Refer to TIDEP-0091 Design Guide for correct .csv format
      fnameGIO = 'scope_x.csv'; % Change this for scope .csv file
      
      %This is calculated by measuring with current probe when power is
      %disconnected
      chan1_zerocurr = 1.22E-02;  %average offset current 
      
      ofname = 'es3_saverest_partcal_meas_pmicJ_initsw_PFM_060518.dat';  %Output Data File
      ofexcelname = 'es3_saverest_partcal_meas_pmicJ_initsw_PFM_060518_out.csv'; %Output .csv file
      chan1_zerocurr = 1.22E-02;  % %average offset current 
      
      %After setting these paramters you can run the script to calculate
      %power consumption measured in Joules. Details can be found in
      %TIDEP-0091 Design Guide
      
   end

   %plotsets = 1; % raw dataset for 5vcurrent, nRST, GPIO_0 graphs
   %plotsets = 2; % above dataset1, using pretrigger to crop data  power graphs
   plotsets = 3; % generates test report

   numstates = 400; % number of detected states (just a test limit)

   pretrigger = 0;   % number samples before trigger
   triggertime = 0.00025;    % time when nRST > 2v 
   % note the trigger for this is column 2 0 -> 1 transition
   % the states are a combination of nRST x1, and GPIO0 x 2

   ofileflag = 1;  % enable output file report
   testtext = 1; % use label text string constants
   % testtext = 2; % read from file
   % testtext = 3; % autonumber, gen out report
   startfig = 1;

   ofilename = strcat(fpath, '\', ofname);
   ofexcelfilename = strcat( fpath, '\', ofexcelname);

   filenameGIO = strcat(fpath,'\', fnameGIO);

   % channel data
   % second, Ch1 curr probe 200ma/div, Ch2 GPIO-0 2v/div, Ch3 nRST 2v/div, Ch4 WARMRESET 2v/div

%  the channel 1 current sensor needs a calibration. 

if(meas_set == 3)
   chan1_zerovolt = 0;

   chan1_calvolt = 1.0;
   chan1_calcurr = 1.0; 

   curr5v_calfactor_gain = 1.0; 
   curr5v_calfactor_bias = -1 * chan1_zerocurr; 
end    
   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % statetext labels are auto only this version

   % read oscilloscope spreadsheet, columns are time(a), and channels(b..)
   timeindex = xlsread(filenameGIO,1, 'A:A');
   numrows = max(size(timeindex))
   numrowsGIO = numrows;

   % the nRST and GPIO0 are states
   nRST = xlsread(filenameGIO,1, 'D:D');
   GPIO0 = xlsread(filenameGIO,1, 'C:C');
   WRESET = xlsread(filenameGIO,1, 'E:E'); 

   % creates the equivalent of a state field
   GPIOM(1: numrows) = 0; % we need to use this array for events
   triggerlevel = 1.0; % the nRST value, or GPIO_0 value converted to states, logic 1 test 
   
   % extract the events
   % nRST x1
   % GPIO0 x 2
   % WRESET x 4
   
   lastval = 0;
   numevents = 1;  % post increment index
   changeflag = 0;
   eventtime(1: numstates) = -99;
   GPIOevent(1: numstates) = 0;
   for n = 1: numrows
   
      if(numevents < (numstates - 1) )  % note, we want to have numstates-2, then end state is numstates-1

         temp1 = 0; 
         if(nRST(n) > triggerlevel);  % test nRST for logic 1
            temp1 = 1.0;
         end
         if(GPIO0(n) > triggerlevel); % test GPIO0 for logic 1
            temp1 = temp1 + 2.0;
         end
         if(WRESET(n) > triggerlevel); % test GPIO0 for logic 1
            temp1 = temp1 + 4.0;
         end

         GPIOM(n) = temp1;
   
         changeflag = 0;
         if(GPIOM(n) < lastval)
            changeflag = 1;
         end   
         if(GPIOM(n) > lastval)
            changeflag = 1;
         end   
         if(changeflag == 1)
            % lastval = GPIO(n);
            lastval = GPIOM(n);
            eventtime(numevents) = timeindex(n);
            GPIOevent(numevents) = GPIOM(n);
            numevents = numevents + 1;
         end   
      end
   end 
   
   disp(sprintf('num masked events %d ', numevents));  

   % read the time index, sampled voltage for current sensor, nRST, GPIO0
   % they go into separate graph variables

   Curr_5v = xlsread(filenameGIO,1, 'B:B');
   Volt_5v(1: numrows) = 5.0;
   % apply current cal to input data
   Curr_5v = (Curr_5v * curr5v_calfactor_gain) + curr5v_calfactor_bias;

   x = 1: 1: numrows;  % assumes we can use GPIO-time

   if(plotsets == 1)
   % plots of data in sections, including time offsets
      figure(startfig);
      startfig = startfig + 1;
      plot(timeindex , GPIOM, 'blue');
      title('x Time sec, y GPIO(nRST x1, GPIO0x2, WRESETx4) - blue ');   % we have 3state bits
      xlabel('time index relative to start sec');
      ylabel('GPIO(masked) logic output');
      grid on;
      zoom on;

      % plot of GPIOM, 5v current
      figure(startfig);
      startfig = startfig + 1;
      plot(timeindex , GPIOM, 'green', timeindex, Volt_5v, 'blue', timeindex, Curr_5v*10, 'red');
      title('x Time sec, y GPIO(nRST x1, GPIO0x2, WRESETx4), Volt5v - blue, Curr5v*10 - red');
      xlabel('time index relative to start sec');
      ylabel('GPIO(nRST x1, GPIO0x2), volt 5v, Curr 5v*10');
      grid on;
      zoom on;

   end

   if(plotsets == 2)  % these are plots over a cropped section, and include calculated current by rail
      triggerindex = findindextime(timeindex, triggertime,  pretrigger)
      startplot = triggerindex;
      stopplot = numrows;
   
      if(triggerindex > 1)
         
         figure(startfig);
         startfig = startfig + 1;

         plot(timeindex(startplot:stopplot), GPIOM(startplot:stopplot), 'blue');
         title('Time, GPIO(masked) - blue');
         xlabel('Time relative to trigger');
         ylabel('GPIO(masked) logic value');
         grid on;
         zoom on;

         % plot of trigger, vs filt voltage rail, and filt current rail
         % 5v rail
         figure(startfig);
         startfig = startfig + 1;
         windowsize = 2;
         filt_Volt5v = filtdata(Volt_5v, windowsize);  %V
         filt_Curr5v = filtdata(Curr_5v, windowsize);  %amps
         
         plot(timeindex(startplot:stopplot), GPIOM(startplot:stopplot), 'green', timeindex(startplot:stopplot), filt_Volt5v(startplot:stopplot), 'blue', timeindex(startplot:stopplot), filt_Curr5v(startplot:stopplot), 'red');
         title('Time, GPIO(masked)- green, filtered Volt 5v - blue, filtered Curr 5v - red');
         xlabel('Time relative to trigger');
         ylabel(' GPIO(masked), filt5v voltage, filt5v Current');
         grid on;
         zoom on

         % plot of trigger, vs filt power rail
         figure(startfig);
         startfig = startfig + 1;
         filtpower5v(1: numrows) = 1.0;
         for n = 1: numrows
            filtpower5v(n) = filt_Volt5v(n) * filt_Curr5v(n) * 1000;  %mW
         end   

         plot(timeindex(startplot:stopplot), GPIOM(startplot:stopplot) *100, 'green', timeindex(startplot:stopplot), filtpower5v(startplot:stopplot), 'red');
         title('Time, GPIO(masked)*100 - green, filtered 5v Power mw - red');
         xlabel('Time relative to trigger');
         ylabel(' GPIO(masked) * 100, filt.5v power mw');
         grid on;
         zoom on
         
         clear filt_Volt5v;
         clear filt_Curr5v;
      end                  
   end    % end of plot set 2
   
   if(plotsets == 3)
      triggerindex = findindextime(timeindex, triggertime,  pretrigger);
      startplot = triggerindex;
      stopplot = numrows;
      
      % this section creates a data file, that searches the database for
      % timing states, then it creates a report file that has time, average
      % voltage, average current, power over the average
      
      startindex = findindextime(timeindex, triggertime,  0); % pass the first trigger index as a seed
      
      % here is a change to use a masked GPIO instead of raw GPIO..
      % [numstatearr statevalarr] = findstateptsGPIO(GPIO0, GPIO1, GPIO2, HOSTINT, startindex, numstates);
      [numstatearr statevalarr] = findstateptsGPIOMasked(GPIOM, startindex, numstates);
      
      detstate = 0;
      for n = 1: numstates
         if(numstatearr(n) > 0)
            detstate = n;
         end   
      end
      % add one state for the end
      detstate = detstate + 1;
      
      if(ofileflag == 1)
         fid = fopen(ofilename, 'wt');
         fid2 = fopen(ofexcelfilename, 'wt');
         
         %1st report file header
         fprintf(fid, 'Test report plot_5vcur_nRST_GPIO0.m \n');
         fprintf( fid, sprintf(' source Excel file for measurement %s \n', fnameGIO ) );
         fprintf(fid, sprintf('num GPIO states programmed %d, num states detected %d \n', numstates, detstate));

         %2nd file header
         CSVstr = sprintf('Test num, Duration, starttime, GPIOm, 5v cur, 5v pow \n');
         fprintf(fid2, '%s',CSVstr);
         
         total5vcurr = 0.0;

         total5vpower = 0.0;

         totalDuration = 0.0;
         
         numvalloops = 0;
         for n = 1: detstate
            startindex = numstatearr(n);
            if( n+1 <= numstates)   
               endindex = numstatearr(n+1);
            else
               endindex = numstatearr(n); 
            end
            
            if(endindex > 0)
               fprintf(fid, sprintf(' detstate# %d, Excelrow# %d, Stateval# %x \n', n, numstatearr(n), statevalarr(n) ));

               % calculate time
               timed_event = timeindex(endindex) - timeindex(startindex);
               % calculate avg voltage, avg current, power over event each
               % rail
               avgvolt5v = mean(Volt_5v(startindex: endindex));
               avgCurr5v = mean(Curr_5v(startindex: endindex));
               peakCurr5v = max(Curr_5v(startindex: endindex));
               avgPowerT5v = avgvolt5v * avgCurr5v * timed_event;
           
               statetext = sprintf('Test# %d', n);
                
               fprintf(fid, sprintf(' eventnum %d, event name - %s \n', n,  statetext));
               fprintf(fid, sprintf(' Duration %f, starttime %f, endtime %f \n',timed_event, timeindex(startindex), timeindex(endindex)) );
               fprintf(fid, sprintf(' 5v avg voltage %f, 5v peak current %f, 5v avg current %f, 5v avg power %f \n',avgvolt5v, peakCurr5v, avgCurr5v, avgPowerT5v));
               
               % special CSV format file for output file 2
               % CSVstr = sprintf('Test num, Duration, starttime, GPIOval, 5v cur, 5v pow');
               CSVdstr = sprintf('%s, %f, %f, %d, %f, %f \n',statetext, timed_event, timeindex(startindex), statevalarr(n), avgCurr5v, avgPowerT5v);     
               fprintf(fid2, '%s',CSVdstr);
               
               % integrate total current and total power
               totalDuration = totalDuration + timed_event;
               total5vcurr = total5vcurr + avgCurr5v;
               total5vpower = total5vpower + avgPowerT5v;

               numvalloops = numvalloops + 1; 
            end % valid index 
         end % loop   
               
         fprintf(fid, sprintf(' Total Duration %f \n', totalDuration) );
         if(numvalloops > 0)
            fprintf(fid, sprintf(' 5v total avg current %f, 5v total avg power %f \n',total5vcurr/numvalloops, total5vpower));

            CSVstr2 = sprintf('\nTotalDuration, 5v total pwr \n');
            fprintf(fid2, '%s',CSVstr2);
            CSVdstr2 = sprintf('%f, %f \n',totalDuration, total5vpower);     
            fprintf(fid2, '%s',CSVdstr2);
         end
            
         fclose(fid); 
         fclose(fid2);
      end % end file

   end %end plotset3    

return;

function dataout = filtdata(datain, windowsize)
   if(windowsize > 1)
      if(max(size(datain)) > windowsize) 
          
         % use filter function for moving average filter
         a = 1;
         b = (1/windowsize)*ones(1, windowsize);
         
         dataout = filter(b, a, datain);
      else
         dataout = 0;
      end
   else
      dataout = 0;
   end
return   

function triggerindex = findindextime(timein, timeref,  pretrigger)
% find the first positive sample in the array
   triggerindex = 1;
   numdata = max(size(timein));
   
   if( numdata > pretrigger)
       index = 0;
       for n = 1: numdata
          if(index == 0)
              if(timein(n) >= timeref)
                 index = n;
              end
          end
       end
       triggerindex = index - pretrigger;
   end
return 

function [numstatearr statevalarr] = findstateptsGPIO(GPIO0, GPIO1, GPIO2, HOSTINT, startindex, numstates) 
   % create a state trigger arraynum
   % purpose is to detect GPIO state transitions in the recorded data, and
   % create indexes for auto integration
   numrows = max(size(GPIO0));
   triggerstate(1: numrows) = 0;
   statevalarr(1: numstates) = 0;
   numstatearr(1: numstates) = 0;
   stateflag = 1;
   index = 1;
   initialflag = 1;
   ncompareflag = 0;
   
   for n = startindex - 1: numrows
      % note GPIO values are set to 0 or 3v 
      triggerstate(n) = (GPIO0(n)/3) + (GPIO1(n)*2/3) + (GPIO2(n)*4/3) + (HOSTINT(n) * 8/3);
      if(initialflag == 1)
         statevalarr(index) = triggerstate(n); % record initial state 1 step before trigger
         numstatearr(index) = n + 1; 
         index = index + 1;
         initialflag = 0;
      else
         ncompareflag = 0; 
         temp = triggerstate(n);
         if(statevalarr(index - 1) > temp ) % record state differences
            ncompareflag = 1;
         else
            if(statevalarr(index - 1) < temp)  
               ncompareflag = 1;
            end
         end
         
         if(ncompareflag == 1)
            if(stateflag == 1)
               numstatearr(index) = n + 1;
               statevalarr(index) = temp;
               index = index + 1;
               if(index > numstates)  % stop recording state indexes, when the return state[] is full
                  stateflag = 0;
               end
            end 
         end
      end   
   end
return   

function [numstatearr statevalarr] = findstateptsGPIOMasked(GPIOM, startindex, numstates) 
   % create a state trigger arraynum
   % purpose is to detect GPIO state transitions in the recorded data, and
   % create indexes for auto integration
   numrows = max(size(GPIOM));
   triggerstate(1: numrows) = 0;
   statevalarr(1: numstates) = 0;
   numstatearr(1: numstates) = 0;
   stateflag = 1;
   index = 1;
   initialflag = 1;
   ncompareflag = 0;
   
   for n = startindex - 1: numrows
      % uses masked array instead of raw GPIO 
      triggerstate(n) = GPIOM(n);
      if(initialflag == 1)
         statevalarr(index) = triggerstate(n); % record initial state 1 step before trigger
         numstatearr(index) = n + 1; 
         index = index + 1;
         initialflag = 0;
      else
         ncompareflag = 0; 
         temp = triggerstate(n);
         if(statevalarr(index - 1) > temp ) % record state differences
            ncompareflag = 1;
         else
            if(statevalarr(index - 1) < temp)  
               ncompareflag = 1;
            end
         end
         
         if(ncompareflag == 1)
            if(stateflag == 1)
               numstatearr(index) = n + 1;
               statevalarr(index) = temp;
               index = index + 1;
               if(index > numstates)  % stop recording state indexes, when the return state[] is full
                  stateflag = 0;
               end
            end 
         end
      end   
   end
return 
      
         
