###################################################################################
#   IWR1443 makefile for Level Sensing Demo
###################################################################################
.PHONY: IWRdemo IWRdemoClean

###################################################################################
# Setup the VPATH:
###################################################################################
vpath %.c ./source
vpath %.h ./include

R4F_INCLUDE += -i$(MMWAVE_SDK_INSTALL_PATH)/ti/control/mmwavelink/test/common	\
			   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/control/mmwavelink/include	 \
			   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/control/mmwavelink

###################################################################################
# Additional libraries which are required to build the executable:
###################################################################################
STD_LIBS = $(R4F_COMMON_STD_LIB) 					\
			 -llibpinmux_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibdma_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibadcbuf_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibhwa_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibmailbox_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibedma_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibmmwave_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibmmwavelink_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibcrc_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibspi_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 	\
			 -llibuart_$(MMWAVE_SDK_DEVICE_TYPE).$(R4F_LIB_EXT) 
LOC_LIBS = $(R4F_COMMON_LOC_LIB)	 \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/pinmux/lib  -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/dma/lib \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/adcbuf/lib  -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/hwa/lib \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/mailbox/lib -i$(MMWAVE_SDK_INSTALL_PATH)/ti/control/mmwave/lib \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/crc/lib     -i$(MMWAVE_SDK_INSTALL_PATH)/ti/control/mmwavelink/lib \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/spi/lib     -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/uart/lib \
   -i$(MMWAVE_SDK_INSTALL_PATH)/ti/drivers/edma/lib 

###################################################################################
# Demo Files
###################################################################################
DEMO_CFG	 = ./mss.cfg
R4_CONFIGPKG = ./mss_configPkg_$(MMWAVE_SDK_DEVICE)
R4_MAP       = ./Debug/level_sense_demo.map
R4_OUT       = ./Debug/level_sense_demo.out
R4_BIN       = ./Debug/level_sense_demo.bin
R4_APP_CMD	 = ./r4_linker.cmd
R4_SOURCES   = ./main.c ./source/demo.c ./source/iwr_gpio.c
R4_DEPENDS   = $(addprefix $(PLATFORM_OBJDIR)/, $(R4_SOURCES:.c=.$(R4F_DEP_EXT)))
R4_OBJECTS   = $(addprefix $(PLATFORM_OBJDIR)/, $(R4_SOURCES:.c=.$(R4F_OBJ_EXT)))

###################################################################################
# RTSC Configuration:
###################################################################################
rtsc_cfg: $(R4_CFG)
	@echo 'Configuring RTSC packages...'
	$(XS) --xdcpath="$(XDCPATH)" xdc.tools.configuro $(R4F_XSFLAGS) -o $(R4_CONFIGPKG) $(DEMO_CFG)
	@echo 'Finished configuring packages'
	@echo ' '

###################################################################################
# Build R4F executable:
###################################################################################
IWRdemo: BUILD_CONFIGPKG=$(R4_CONFIGPKG)
IWRdemo: R4F_CFLAGS += --cmd_file=$(BUILD_CONFIGPKG)/compiler.opt
IWRdemo: buildDirectories rtsc_cfg $(R4_OBJECTS)
	$(R4F_LD) $(R4F_LDFLAGS) $(LOC_LIBS) $(STD_LIBS) 	\
	-l$(R4_CONFIGPKG)/linker.cmd --map_file=$(R4_MAP) 		\
	$(R4_OBJECTS) $(PLATFORM_R4F_LINK_CMD) $(R4_APP_CMD) 		\
	$(R4F_LD_RTS_FLAGS) -o $(R4_OUT)
	@echo '************************************************************************'
	@echo 'Built the IWR1443 MSS Level Sensing Demo executable'
	@echo '************************************************************************'

###################################################################################
# R4 Demo Cleanup:
###################################################################################
IWRdemoClean:
	@echo 'Cleaning the R4F demo objects'
	@$(DEL) $(R4_OBJECTS) $(R4_OUT) $(R4_BIN)
	@$(DEL) $(R4_MAP) $(R4_DEPENDS)
	@echo 'Cleaning the R4F demo RTSC package'
	@$(DEL) $(R4_CONFIGPKG)
	@$(DEL) $(PLATFORM_OBJDIR)

# Dependency handling
-include $(R4_DEPENDS)

