/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef __DEMO_H__
#define __DEMO_H__

#include <ti/common/mmwave_error.h>
#include <ti/drivers/soc/soc.h>
#include <ti/drivers/uart/UART.h>
#include <ti/drivers/pinmux/pinmux.h>
#include <ti/drivers/esm/esm.h>
#include <ti/drivers/soc/soc.h>
#include <ti/drivers/mailbox/mailbox.h>
#include <ti/drivers/adcbuf/ADCBuf.h>
#include <ti/drivers/hwa/hwa.h>
#include <ti/drivers/edma/edma.h>
#include <ti/control/mmwave/mmwave.h>
#include <ti/control/mmwavelink/include/rl_sensor.h>

#include <ti/sysbios/knl/Semaphore.h>
#include <ti/common/sys_common_xwr14xx.h>

#include "../../msp_level_sense_demo/demo_common.h"


#ifdef DEBUG
#define demo_printf(...)               System_printf(__VA_ARGS__)
#else
#define demo_printf(...)               //
#endif

#define DEMO_RUN_MEASUREMENT           0
#define DEMO_RUN_CALIBRATION           1
#define DEMO_GOOD_CAL_RESULT           0x17fe

//Choose one configurations defined below
#define DEMO_CONFIG15
//#define DEMO_CONFIG_TEST

/* Define chirp configurations for the demo */

#ifdef  DEMO_CONFIG0 //these are the mmwave demo parameters
//Description: Short range with good resolution. Low processing.
//max range: 4.5m
//range resolution: 4cm
#define DEMOCFG_SAMPLE_RATE            4500        /* 4.5Msps */
#define DEMOCFG_FREQ_SLOPE             1408        /* x 48.28kHz/us = 68Mhz/us */
#define DEMOCFG_IDLE_TIME              700         /* 7us */
#define DEMOCFG_ADC_START_TIME         700         /* 7us */
#define DEMOCFG_RAMP_END_TIME          5800        /* 58us */
#define DEMOCFG_RANGE_RES              0.038787879
#define DEMOCFG_NUM_ADC_SAMPLES        225
#define DEMOCFG_FFT_SIZE               256
#endif

#ifdef  DEMO_CONFIG1
//Description: Short range with good resolution. Low processing.
//max range: 4.5m
//range resolution: 3cm
#define DEMOCFG_SAMPLE_RATE            4500        /* 4.5Msps */
#define DEMOCFG_FREQ_SLOPE             930         /* x 48.28kHz/us = 45Mhz/us */
#define DEMOCFG_IDLE_TIME              1500        /* 15us */
#define DEMOCFG_ADC_START_TIME         400         /* 4us */
#define DEMOCFG_RAMP_END_TIME          8900        /* 89us */
#define DEMOCFG_RANGE_RES              0.0294
#define DEMOCFG_NUM_ADC_SAMPLES        376
#define DEMOCFG_FFT_SIZE               512
#endif

#ifdef  DEMO_CONFIG2
//Description: Medium range with good resolution. Higher processing.
//max range: 30m
//range resolution: 3.4cm
#define DEMOCFG_SAMPLE_RATE            7500        /* 7.5Msps */
#define DEMOCFG_FREQ_SLOPE             592         /* x 48.28kHz/us = 28.5MHz/us */
#define DEMOCFG_IDLE_TIME              1600        /* 16us */
#define DEMOCFG_ADC_START_TIME         700         /* 7us */
#define DEMOCFG_RAMP_END_TIME          13933       /* 139us */
#define DEMOCFG_RANGE_RES              0.0385
#define DEMOCFG_NUM_ADC_SAMPLES        985
#define DEMOCFG_FFT_SIZE               1024
#endif

#ifdef  DEMO_CONFIG3
//Description: Long range with okay resolution. Low processing.
//max range: 60m
//range resolution: 6.6cm
#define DEMOCFG_SAMPLE_RATE            5000        /* 5.0Msps */
#define DEMOCFG_FREQ_SLOPE             462         /* x 48.28kHz/us = 22.3Mhz/us */
#define DEMOCFG_IDLE_TIME              1600        /* 16us */
#define DEMOCFG_ADC_START_TIME         700         /* 7us */
#define DEMOCFG_RAMP_END_TIME          9200        /* 92us */
#define DEMOCFG_RANGE_RES              0.0658
#define DEMOCFG_NUM_ADC_SAMPLES        421
#define DEMOCFG_FFT_SIZE               512
#endif



//AG: New set of configs from Joe's Master Spreadsheet.


#ifdef DEMO_CONFIG15
    #define DEMOCFG_SAMPLE_RATE             7500    //  profileCfg->digOutSampleRate
    #define DEMOCFG_FREQ_SLOPE              590 //  profileCfg->freqSlopeConst
    #define DEMOCFG_ADC_START_TIME              700 //  profileCfg->adcStartTimeConst
    #define DEMOCFG_EXCESS_RAMP_TIME                100 //
    #define DEMOCFG_IDLE_TIME               700 //  profileCfg->idleTimeConst
    #define DEMOCFG_NUM_ADC_SAMPLES             990 //  profileCfg->numAdcSamples
    #define DEMOCFG_RAMP_END_TIME               14000   //  profileCfg->rampEndTime
    #define DEMOCFG_FFT_SIZE                1024    //  dataPathObj->numRangeBins
    #define DEMOCFG_NUMLOOPS                2   //  frameCfg->numLoops
    #define DEMOCFG_START_FREQCONST             1435384036  //  profileCfg->startFreqConst
    #define DEMOCFG_RxLNAGAIN               30  //  profileCfg->rxGain
    #define DEMOCFG_NONCOHERENT_COMBINING               2   //  future parameter, average last FFT s
    #define DEMOCFG_RANGE_RES               0.03758776  //  dataPathObj->rangeResolution
    #define DEMOCFG_ZOOMFFT             0   //  future parameter
    #define DEMOCFG_TXUSED              1   //  2^(value - 1) - value is a mask, value is antenna# 1,2,3
    #define DEMOCFG_RXUSED              3   //  2^(value - 1) - value is a mask, value is antenna# 1,2,3,4
    #define DEMOCFG_TxBackoff               0   //  value is numdb, and used Tx, bit7.0Tx1, bit15.8Tx2, bit23.16 Tx3
#endif


#ifdef DEMO_CONFIG30
    #define DEMOCFG_SAMPLE_RATE             7500    //
    #define DEMOCFG_FREQ_SLOPE              590 //
    #define DEMOCFG_ADC_START_TIME              700 //
    #define DEMOCFG_EXCESS_RAMP_TIME                100 //
    #define DEMOCFG_IDLE_TIME               700 //
    #define DEMOCFG_NUM_ADC_SAMPLES             990 //
    #define DEMOCFG_RAMP_END_TIME               14000   //
    #define DEMOCFG_FFT_SIZE                1024    //
    #define DEMOCFG_NUMLOOPS                5   //  frameCfg->numLoops
    #define DEMOCFG_START_FREQCONST             1.4353840356E+09    //  profileCfg->startFreqConst
    #define DEMOCFG_RxLNAGAIN               34  //  profileCfg->rxGain
    #define DEMOCFG_NONCOHERENT_COMBINING               5   //
    #define DEMOCFG_RANGE_RES               0.038   //
    #define DEMOCFG_ZOOMFFT             0   //  future parameter
    #define DEMOCFG_TXUSED              1   //  is the actual antenna#
    #define DEMOCFG_RXUSED              3   //  is the actual antenna#
    #define DEMOCFG_TxBackoff               0   //  needs to be the combined Tx1, Tx2, Tx3
#endif

#define DEMO_CAL_FREQ_LOW              770
#define DEMO_CAL_FREQ_HIGH             810
#define DEMO_CAL_FLASH_OFFSET          0x100000
#define DEMO_CAL_FLASH_LEN            (sizeof(rlCalDataStore_t) * RL_MAX_CALIB_DATA_CHUNK)

#define DEMOCFG_RANGE_BINS                 (DEMOCFG_NUM_ADC_SAMPLES / 2)

#define DEMOCFG_NUM_CHIRPS                  1 /* number of collected chirps to process */
#define DEMOCFG_NUM_CHIRP_CFG               1 /* number of different chirp configs */

#define HWAUTIL_NUM_PARAM_SETS_1D           4
#define DEMOCFG_SPEED_OF_LIGHT_IN_METERS   (3.0e8)


#define SOC_AR14XX_MSS_L3RAM_SIZE           0x40000
#define PI                                  3.14159265
#define ONE_Q17                            (1U << 17)
#define MMW_HWA_START_POS_PARAMSETS_1D      0
#define MMW_HWA_WINDOWRAM_1D_OFFSET         0 //In samples

#define MMW_HWA_DMA_TRIGGER_SOURCE_1D_PING  0
#define MMW_HWA_DMA_DEST_CHANNEL_1D_PING    0
#define MMW_HWA_1D_OUT_PING                (&gMmwHwaMemBuf[2])

#define MMW_HWA_DMA_TRIGGER_SOURCE_1D_PONG  1
#define MMW_HWA_DMA_DEST_CHANNEL_1D_PONG    1
#define MMW_HWA_1D_OUT_PONG                (&gMmwHwaMemBuf[3])

#define EDMA_SHADOW_LNK_PARAM_BASE_ID       EDMA_NUM_DMA_CHANNELS

/* 1D */
#define MMW_HWA_1D_ADCBUF_INP              (&gMmwHwaMemBuf[0])

/* 1D -ping */
#define MMW_HWA_DMA_TRIGGER_SOURCE_1D_PING  0
#define MMW_HWA_DMA_DEST_CHANNEL_1D_PING    0
#define MMW_HWA_1D_OUT_PING                (&gMmwHwaMemBuf[2])
#define MMW_EDMA_1D_PING_CH_ID              EDMA_TPCC0_REQ_HWACC_0
#define MMW_EDMA_1D_PING_SHADOW_LINK_CH_ID  EDMA_SHADOW_LNK_PARAM_BASE_ID
#define MMW_EDMA_1D_PING_CHAIN_CH_ID        EDMA_TPCC0_REQ_FREE_0
#define MMW_EDMA_1D_PING_ONE_HOT_SHADOW_LINK_CH_ID (EDMA_SHADOW_LNK_PARAM_BASE_ID + 2)

/* 1D - pong */
#define MMW_HWA_DMA_TRIGGER_SOURCE_1D_PONG   1
#define MMW_HWA_DMA_DEST_CHANNEL_1D_PONG     1
#define MMW_HWA_1D_OUT_PONG                 (&gMmwHwaMemBuf[3])
#define MMW_EDMA_1D_PONG_CH_ID               EDMA_TPCC0_REQ_HWACC_1
#define MMW_EDMA_1D_PONG_SHADOW_LINK_CH_ID  (EDMA_SHADOW_LNK_PARAM_BASE_ID + 1)
#define MMW_EDMA_1D_PONG_CHAIN_CH_ID         EDMA_TPCC0_REQ_FREE_1
#define MMW_EDMA_1D_PONG_ONE_HOT_SHADOW_LINK_CH_ID (EDMA_SHADOW_LNK_PARAM_BASE_ID + 3)

#define DEMO_EDMA_TRANSFER_COMPLETION_CODE_1D_DONE   MMW_EDMA_1D_PONG_CHAIN_CH_ID


#define MMW_HWA_NUM_MEM_BUFS 4


typedef struct
{
    uint8_t buf[SOC_XWR14XX_MSS_HWA_MEM_SIZE/MMW_HWA_NUM_MEM_BUFS];
} mmwHwaBuf_t;


/*! simple memory pool structure for flexible partitioning of L3 RAM */
typedef struct
{
    /*! base address of memory */
    uint8_t *base;

    /*! size of memory in bytes */
    uint32_t size;

    /*! index into the pool to track where is next free */
    uint32_t indx;
} DemoMemPool_t;


#ifdef RANGE_DC_CALIBRATION
/**
 * @brief
 *  Millimeter Wave Demo DC range signature compensation
 *
 * @details
 *  The structure contains the DC range antenna signature removeal configuration used in data path
 */
typedef struct Demo_CalibDcRangeSigCfg_t
{
    /*! @brief    enabled flag:  1-enabled 0-disabled */
    uint16_t    enabled;

    /*! @brief    maximum negative range bin (1D FFT index) to be compensated */
    int16_t     negativeBinIdx;

    /*! @brief    maximum positive range bin (1D FFT index) to be compensated */
    int16_t     positiveBinIdx;

    /*! @brief    number of chirps in the averaging phase */
    uint16_t    numAvgChirps;

} Demo_CalibDcRangeSigCfg;
#endif


/*!
 *  @brief      Data path mode
 *
 *  This enum defines if the data path chain is running standalone (such as in a unit test)
 *  or if it is running as part of a demo (integrated with ADC buffer)
 */
typedef enum
{
    /*!
      *  Data path running standalone
      */
    DATA_PATH_STANDALONE,
    /*!
      *  Data path integrated with ADC buffer
      */
    DATA_PATH_WITH_ADCBUF
} DataPath_mode;


typedef struct Demo_ADCBufCfg_t
{
    /*! ADCBUF out format:
        0-Complex,
        1-Real */
    uint8_t     adcFmt;

    /*! ADCBUF IQ swap selection:
        0-I in LSB, Q in MSB,
        1-Q in LSB, I in MSB */
    uint8_t     iqSwapSel;

    /*! ADCBUF channel interleave configuration:
        0-interleaved(not supported on XWR16xx),
        1- non-interleaved */
    uint8_t     chInterleave;

    /**
     * @brief   Chirp Threshold configuration used for ADCBUF buffer
     */
    uint8_t     chirpThreshold;
}Demo_ADCBufCfg;


/**
 * @brief
 *  Millimeter Wave Demo Data Path Information.
 *
 * @details
 *  The structure is used to hold all the relevant information for
 *  the data path.
 */
typedef struct Demo_DataPathObj_t
{
    /*! @brief   Number of receive channels */
    uint32_t numRxAntennas;

    /*! @brief   Element size (in number of bytes) used to configure DMA channel */
    uint16_t dmaElemSize;

    /*! @brief   Frame count used to configure DMA channel */
    uint8_t dmaFrameCnt;

    /*! @brief   Counter which tracks the number of chirps detected */
    uint32_t chirpInterruptCounter;

    /*! @brief   ADCBUF handle. */
    ADCBuf_Handle adcbufHandle;

    /*! @brief   Handle of the EDMA driver. */
    EDMA_Handle edmaHandle;

    /*! @brief   EDMA error Information when there are errors like missing events */
    EDMA_errorInfo_t  EDMA_errorInfo;

    /*! @brief EDMA transfer controller error information. */
    EDMA_transferControllerErrorInfo_t EDMA_transferControllerErrorInfo;

    /*! @brief Semaphore handle for 1D EDMA completion. */
    Semaphore_Handle EDMA_1Ddone_semHandle;

    /*! @brief Handle to hardware accelerator driver. */
    HWA_Handle  hwaHandle;

    /*! @brief Semaphore handle for Hardware accelerator completion. */
    Semaphore_Handle HWA_done_semHandle;

    /*! @brief Hardware accelerator Completion Isr count for debug purposes. */
    uint32_t hwaDoneIsrCounter;

    /*! @brief Semaphore handle for Frame start indication. */
//    Semaphore_Handle frameStart_semHandle;

    /*! @brief Hardware accelerator Completion Isr count for debug purposes. */
//    uint32_t frameStartIntCounter;

    /*! @brief  Used for checking that inter frame processing finshed on time */
//    int32_t interFrameProcToken;

    /*! @brief noise energy */
    uint32_t noiseEnergy;

    /*! @brief Pointer to Radar Cube memory in L3 RAM */
    uint32_t *radarCube;

    /*! @brief Pointer to range-doppler log magnitude matrix memory in L3 RAM */
    uint16_t *rangeDopplerLogMagMatrix;

    /*! @brief number of transmit antennas */
    uint32_t numTxAntennas;

    /*! @brief number of virtual antennas */
    uint32_t numVirtualAntennas;

    /*! @brief number of virtual azimuth antennas */
    uint32_t numVirtualAntAzim;

    /*! @brief number of virtual elevation antennas */
    uint32_t numVirtualAntElev;

    /*! @brief number of ADC samples */
    uint32_t numAdcSamples;

    /*! @brief number of range bins */
    uint32_t numRangeBins;

    /*! @brief number of chirps per frame */
    uint32_t numChirpsPerFrame;

    /*! @brief number of doppler bins */
    uint32_t numDopplerBins;

    /*! @brief range resolution in meters */
    float rangeResolution;

    /*! @brief Data path mode */
    DataPath_mode dataPathMode;

    /*! @brief status of boot calibration */
    uint32_t calibrationStatus;

    #ifdef RANGE_DC_CALIBRATION
    /*! @brief   Calibrate DC (zero) range signiture */
    Demo_CalibDcRangeSigCfg calibDcRangeSigCfg;

    /*! @brief Pointer to DC range signature compensation buffer */
    cmplx32ImRe_t *dcRangeSigMean;

    /*! @brief DC range signature calibration counter */
    uint32_t dcRangeSigCalibCntr;

    /*! @brief log2 of number of averaged chirps */
    uint32_t log2NumAvgChirps;
    #endif

/*! @brief Semaphore handle for Hardware accelerator completion. */
//Semaphore_Handle frameStart_semHandle;

} Demo_DataPathObj;


/** @brief Configures ParameterSet for Range FFT on ADC samples from DFE
 *
 *   @param[in] handle                  HWA driver handle
 *
 *   @param[in] paramSetStartIdx        HWA parameter set start index
 *
 *   @param[in] numAdcSamples           Number of ADC samples
 *
 *   @param[in] numRangeBins            Number of range bins (1st D FFT size)
 *
 *   @param[in] numRxAnt                Number of Rx antennas
 *
 *   @param[in] windowOffsetBytes       Window offset in bytes (for pre-FFT windowing)
 *
 *   @param[in] dmaTriggerSourcePing    DMA trigger source channel for Ping param sets
 *
 *   @param[in] dmaTriggerSourcePong    DMA trigger source channel for Pong param sets
 *
 *   @param[in] dmaDestChannelPing      DMA destination channel for Ping param sets
 *
 *   @param[in] dmaDestChannelPong      DMA destination channel for Pong param sets
 *
 *   @param[in] hwaMemAdcBufOffset      HWA memory offset ADC buffer
 *
 *   @param[in] hwaMemDestPingOffset    HWA memory offset for Ping destination
 *
 *   @param[in] hwaMemDestPongOffset    HWA memory offset for Pong destination
 *
 */
void HWAutil_configRangeFFT(HWA_Handle handle,
                            uint32_t  paramSetStartIdx,
                            uint32_t numAdcSamples,
                            uint32_t numRangeBins,
                            uint8_t numRxAnt,
                            uint32_t windowOffsetBytes,

                            uint8_t dmaTriggerSourcePing,
                            uint8_t dmaTriggerSourcePong,

                            uint8_t dmaDestChannelPing,
                            uint8_t dmaDestChannelPong,

                            uint16_t hwaMemAdcBufOffset,

                            uint16_t hwaMemDestPingOffset,
                            uint16_t hwaMemDestPongOffset,
                            uint8_t  hwaTriggerMode);


/**
 * @brief
 *  Industrial Level Sensing Demo MCB
 *
 * @details
 *  The structure is used to hold all the relevant information for the
 *  Industrial Level Sensing demo.
 */
typedef struct LSDemo_MCB_t
{
    /*! * @brief   Handle to the SOC Module */
    SOC_Handle                  socHandle;

    /*! @brief   This is the mmWave control handle which is used
     * to configure the BSS. */
    MMWave_Handle               ctrlHandle;

    /*! @brief   mmWave Control Configuration. */
    MMWave_CtrlCfg              ctrlCfg;

    /*! @brief   mmWave Open Configuration. */
    MMWave_OpenCfg              openCfg;

    /*! @brief   mmWave Profile Configuration. */
    rlProfileCfg_t              profileCfg;

    /*! @brief   mmWave Chirp Configuration. */
    MMWave_ChirpHandle          chirpHandle;
    rlChirpCfg_t                chirpCfg;

    /*! @brief   mmWave Calibration Configuration. */
    MMWave_CalibrationCfg       calCfg;

    /*! @brief   ADCBuf Configuration. */
    Demo_ADCBufCfg              adcbufCfg;

    /*! @brief   Data Path object */
    Demo_DataPathObj            dataPathObj;
} LSDemo_MCB;


void demo_dataPathInit(Demo_DataPathObj *obj);
void demo_dataPathOpen(Demo_DataPathObj *obj);
void demo_dataPathConfig (void);
void demo_dataPathConfigCommon(Demo_DataPathObj *obj);

void demo_dataPathTask(UArg arg0, UArg arg1);
void demo_mmWaveCtrlTask(UArg arg0, UArg arg1);
void demo_calibrationTask(UArg arg0, UArg arg1);

#endif // __DEMO_H__
