/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
/*******************************************************************************
 *    IWR1443+MSP432 Industrial Fluid Level Sensing Demo
 *
 *    This code builds the MSP432 executable.  The IWR1443
 *    source is in the ./level_sensing_demo/iwr1443 directory.
 * --------------------------------------------------------
 *
 * This file contains the mmWave setup and control functions for the demo.
 *
 *******************************************************************************/
/* Standard Includes */
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <math.h>

#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

#include <ti/common/sys_common.h>
#include <ti/drivers/soc/soc.h>
#include <ti/drivers/esm/esm.h>
#include <ti/drivers/mailbox/mailbox.h>
#include <ti/drivers/hwa/hwa.h>
#include <ti/drivers/spi/SPI.h>
#include <ti/drivers/qspiflash/qspiflash.h>

#include <ti/control/mmwave/mmwave.h>
#include <ti/control/mmwavelink/mmwavelink.h>
#include <ti/control/mmwave/include/mmwave_internal.h>
#include "rl_datatypes.h"
#include "rl_sensor.h"
#include "rl_driver.h"

/* Demo Includes */
#include "demo.h"
#include "iwr_gpio.h"


/* Local compile-time options */

//CZ_BOARD should be defined when running on the CZ board, especially with
//waveguide, since the Tx/Rx power requirements are different. This should
//not be defined for normal use.
//#define CZ_BOARD

//When RUN_ONESHOT_CAL is defined, the demo will command the BSS to run
//one-shot runtime cal prior to the chirps. This is the default, to update
//the full cal data that is stored in flash. If running full cal, this does
//have to run.
#define RUN_ONESHOT_CAL

//When POWER_TEST is defined, the GPIO_0 toggles will not match what the
//MSP432 expects for measurement power ups. This is disabled by default,
//and will require INCLUDE_CYCLE_PROFILER to be defined as well.
//#define POWER_TEST

//When RUN_FULL_CAL is defined, the configuration process will command
//the BSS to run all boot-time cal tasks, instead of reading data from flash.
//This is disabled by default.  When enabled, it disables flash access.
//#define RUN_FULL_CAL

//When INCLUDE_CYCLE_PROFILER is defined, the Cycle Profiler will be included
//so that you may add calls to capture the number of 200Mhz clocks a certain
//task takes. To get the time in milliseconds, simply subtract two times,
//then divide by 200000.
//#define INCLUDE_CYCLE_PROFILER

#ifdef INCLUDE_CYCLE_PROFILER
#include <ti/utils/cycleprofiler/cycle_profiler.h>
extern unsigned int cptime[12];
#endif

extern SOC_Handle  socHandle;
extern SPI_Handle  spiHandle;

#ifndef RUN_FULL_CAL
extern QSPI_Handle qspiHandle;
extern QSPIFlash_Handle qspiflashHandle;
#endif


/* Globals */
LSDemo_MCB    gMmwMCB;

rlCalibrationData_t init_cal_data[RL_MAX_CALIB_DATA_CHUNK];

/*! L3 RAM buffer */
uint8_t gMmwL3[SOC_AR14XX_MSS_L3RAM_SIZE];
#pragma DATA_SECTION(gMmwL3, ".l3ram");

/*! L3 heap for convenicence of partitioning L3 RAM */
DemoMemPool_t gMmwL3heap =
{
    &gMmwL3[0],
    SOC_AR14XX_MSS_L3RAM_SIZE,
    0
};

#pragma DATA_SECTION(gMmwHwaMemBuf, ".hwaBufs");
mmwHwaBuf_t gMmwHwaMemBuf[MMW_HWA_NUM_MEM_BUFS];
#define MMW_HWA_1D_ADCBUF_INP              (&gMmwHwaMemBuf[0])

uint32_t           datapath_err;
LSDemo_ChirpData   chirp_data;
LSDemo_CalData     cal_data;


uint32_t log2Approx(uint32_t x)
{
    uint32_t idx, detectFlag = 0;

    idx = 32U;
    while((detectFlag==0U) || (idx==0U))
    {
        if(x & 0x80000000U)
        {
            detectFlag = 1;
        }
        x <<= 1U;
        idx--;
    }

    if(x != 0)
    {
        idx = idx + 1;
    }

    return(idx);
}


/**
 *  @b Description
 *  @n
 *      Function to generate the whole FFT window Table.
 *
 *  @param[out] win Pointer to output calculated window samples.
 *  @param[in]  winIndx Index of window to generate sample at.
 *  @param[in]  phi Pre-calculated constant by caller as (2*pi/(window length - 1)).
 *  @param[in]  winType Type of window, one of @ref MMW_WIN_BLACKMAN, @ref MMW_WIN_HANNING,
 *              or @ref MMW_WIN_RECT.
 *  @retval none.
 */

/* FFT Window - only *one* of the following must be enabled! (defined) */
#define MMW_WIN_BLACKMAN
//#define MMW_WIN_HANNING
//#define MMW_WIN_RECT

static inline demo_genTable(uint32_t *win, uint32_t numPts, float phi)
{
  uint32_t idx;
  #ifdef MMW_WIN_BLACKMAN
  float a0 = 0.42;
  float a1 = 0.5;
  float a2 = 0.08;
  #endif

  for (idx = 0; idx < numPts; idx ++)
  {
    #ifdef MMW_WIN_BLACKMAN
      *win = (uint32_t) ((ONE_Q17 * (a0 - a1*cos(phi * idx) +
             a2 * cos(2 * phi * idx))) + 0.5); //in Q17

      if (*win >= ONE_Q17)
          *win = ONE_Q17 - 1;
      *win++;
    #endif

    #ifdef MMW_WIN_HANNING
      *win = (uint32_t) ((ONE_Q17 * 0.5* (1 - cos(phi * idx))) + 0.5); //in Q17

      if (*win >= ONE_Q17)
          *win = ONE_Q17 - 1;
      *win++;
    #endif

    #ifdef MMW_WIN_RECT
      *win++ = (uint32_t) (ONE_Q17/16); //in Q17
    #endif
  }
}


#if 0
/**
 *  @b Description
 *  @n
 *      Function to generate a single FFT window sample.
 *
 *  @param[out] win Pointer to output calculated window sample.
 *  @param[in]  winIndx Index of window to generate sample at.
 *  @param[in]  phi Pre-calculated constant by caller as (2*pi/(window length - 1)).
 *  @param[in]  winType Type of window, one of @ref MMW_WIN_BLACKMAN, @ref MMW_WIN_HANNING,
 *              or @ref MMW_WIN_RECT.
 *  @retval none.
 */
static inline demo_genWindow(uint32_t *win, uint32_t winIndx, float phi, uint32_t winType)
{
    if(winType == MMW_WIN_BLACKMAN)
    {
        //Blackman window
        float a0 = 0.42;
        float a1 = 0.5;
        float a2 = 0.08;
        *win = (uint32_t) ((ONE_Q17 * (a0 - a1*cos(phi * winIndx) +
            a2*cos(2 * phi * winIndx))) + 0.5); //in Q17
        if(*win >= ONE_Q17)
        {
            *win = ONE_Q17 - 1;
        }
    }
    else if(winType == MMW_WIN_HANNING)
    {
        //Hanning window
        *win = (uint32_t) ((ONE_Q17 * 0.5* (1 - cos(phi * winIndx))) + 0.5); //in Q17
        if(*win >= ONE_Q17)
        {
            *win = ONE_Q17 - 1;
        }
    }
    else if(winType == MMW_WIN_RECT)
    {
        //Rectangular window
        *win = (uint32_t) (ONE_Q17/16); //in Q17
    }
}
#endif


uint32_t demo_pow2roundup (uint32_t x)
{
    uint32_t result = 1;
    while(x > result)
    {
        result <<= 1;
    }
    return result;
}


/* */
void demo_memPoolReset(DemoMemPool_t *pool)
{
    pool->indx = 0;
}


uint8_t *demo_memPoolAlloc(DemoMemPool_t *pool, uint32_t size)
{
    uint8_t *ptr = NULL;

    if ((size % 4) != 0)
    {
        return(ptr);
    }

    if ((pool->indx + size) <= pool->size)
    {
        ptr = pool->base + pool->indx;
        pool->indx += size;
    }

    return(ptr);
}


/**
 *  @b Description
 *  @n
 *      HWA processing completion call back function as per HWA API.
 *      Depending on the programmed transfer completion codes,
 *      posts HWA done semaphore.
 */
void demo_dataPathHwaDoneIsrCallback(void * arg)
{
    Semaphore_Handle       semHandle;

    gMmwMCB.dataPathObj.hwaDoneIsrCounter++;

    if (arg != NULL) {
        semHandle = (Semaphore_Handle)arg;
        Semaphore_post(semHandle);
    }
}


void HWAutil_configRangeFFT(HWA_Handle handle,
                              uint32_t paramSetStartIdx,
                              uint32_t numAdcSamples,
                              uint32_t numRangeBins,
                               uint8_t numRxAnt,
                              uint32_t windowOffsetBytes,

                               uint8_t dmaTriggerSourcePing,
                               uint8_t dmaTriggerSourcePong,

                               uint8_t dmaDestChannelPing,
                               uint8_t dmaDestChannelPong,

                              uint16_t hwaMemAdcBufOffset,

                              uint16_t hwaMemDestPingOffset,
                              uint16_t hwaMemDestPongOffset,
                               uint8_t hwaTriggerMode)
{
    HWA_InterruptConfig     paramISRConfig;
    int32_t errCode = 0;
    uint32_t paramsetIdx = paramSetStartIdx;
    uint32_t pingParamSetIdx = 0;
    HWA_ParamConfig hwaParamCfg[HWAUTIL_NUM_PARAM_SETS_1D];

    memset(hwaParamCfg,0,sizeof(hwaParamCfg));

    /***********************/
    /* PING DUMMY PARAMSET */
    /***********************/
    hwaParamCfg[paramsetIdx].triggerMode = HWA_TRIG_MODE_DMA; //Software triggered  - in demo this will be HWA_TRIG_MODE_DMA
    hwaParamCfg[paramsetIdx].dmaTriggerSrc = dmaTriggerSourcePing; //in demo this will be first EDMA Src channel id
    hwaParamCfg[paramsetIdx].accelMode = HWA_ACCELMODE_NONE; //dummy
    errCode = HWA_configParamSet(handle,paramsetIdx,&hwaParamCfg[paramsetIdx],NULL);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configParamSet(%d) returned %d\n", paramsetIdx, errCode);
        return;
    }

    /***********************/
    /* PING PROCESS PARAMSET */
    /***********************/
    paramsetIdx++;
    pingParamSetIdx = paramsetIdx;
    hwaParamCfg[paramsetIdx].triggerMode = hwaTriggerMode;
    hwaParamCfg[paramsetIdx].accelMode = HWA_ACCELMODE_FFT; //do FFT

    hwaParamCfg[paramsetIdx].source.srcAddr = hwaMemAdcBufOffset; // address is relative to start of MEM0
    hwaParamCfg[paramsetIdx].source.srcAcnt = numAdcSamples - 1; //this is samples - 1
    hwaParamCfg[paramsetIdx].source.srcAIdx = numRxAnt * sizeof(uint32_t); // 16 bytes
    hwaParamCfg[paramsetIdx].source.srcBcnt = numRxAnt-1; //no iterations here
    hwaParamCfg[paramsetIdx].source.srcBIdx = sizeof(uint32_t); //should be dont care
    hwaParamCfg[paramsetIdx].source.srcShift = 0; //no shift
    hwaParamCfg[paramsetIdx].source.srcCircShiftWrap = 0; //no shift
    hwaParamCfg[paramsetIdx].source.srcRealComplex = HWA_SAMPLES_FORMAT_COMPLEX; //complex data
    hwaParamCfg[paramsetIdx].source.srcWidth = HWA_SAMPLES_WIDTH_16BIT; //16-bit
    hwaParamCfg[paramsetIdx].source.srcSign = HWA_SAMPLES_SIGNED; //signed
    hwaParamCfg[paramsetIdx].source.srcConjugate = 0; //no conjugate
    hwaParamCfg[paramsetIdx].source.srcScale = 8;
    hwaParamCfg[paramsetIdx].source.bpmEnable = 0; //bpm removal not enabled
    hwaParamCfg[paramsetIdx].source.bpmPhase = 0; //dont care

    hwaParamCfg[paramsetIdx].dest.dstAddr = hwaMemDestPingOffset; // address is relative to start of MEM0
    hwaParamCfg[paramsetIdx].dest.dstAcnt = numRangeBins-1; //this is samples - 1
    hwaParamCfg[paramsetIdx].dest.dstAIdx = numRxAnt * sizeof(uint32_t); //
    hwaParamCfg[paramsetIdx].dest.dstBIdx = sizeof(uint32_t); //should be dont care
    hwaParamCfg[paramsetIdx].dest.dstRealComplex = HWA_SAMPLES_FORMAT_COMPLEX; //same as input - complex
    hwaParamCfg[paramsetIdx].dest.dstWidth = HWA_SAMPLES_WIDTH_16BIT; //same as input - 16 bit
    hwaParamCfg[paramsetIdx].dest.dstSign = HWA_SAMPLES_SIGNED; //same as input - signed
    hwaParamCfg[paramsetIdx].dest.dstConjugate = 0; //no conjugate
    hwaParamCfg[paramsetIdx].dest.dstScale = 0;
    hwaParamCfg[paramsetIdx].dest.dstSkipInit = 0; // no skipping

    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.fftEn = 1;
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.fftSize = log2Approx(numRangeBins);
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.butterflyScaling = 0x3; //LSB fftSize bits are relevant - revisit this for all FFT size and data size
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.interfZeroOutEn = 0; //disabled
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.windowEn = 1; //enabled
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.windowStart = windowOffsetBytes; //start of window RAM
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.winSymm = 1; //symmetric - in demo do we make this symmetric
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.winInterpolateMode = 0; //fftsize is less than 1K
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.magLogEn = HWA_FFT_MODE_MAGNITUDE_LOG2_DISABLED; //disabled
    hwaParamCfg[paramsetIdx].accelModeArgs.fftMode.fftOutMode = HWA_FFT_MODE_OUTPUT_DEFAULT; // output FFT samples

    hwaParamCfg[paramsetIdx].complexMultiply.mode = HWA_COMPLEX_MULTIPLY_MODE_DISABLE;
    errCode = HWA_configParamSet(handle,paramsetIdx,&hwaParamCfg[paramsetIdx],NULL);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configParamSet(%d) returned %d\n",errCode,paramsetIdx);
        return;
    }
    /* enable the DMA hookup to this paramset so that data gets copied out */
    paramISRConfig.interruptTypeFlag = HWA_PARAMDONE_INTERRUPT_TYPE_DMA;
    paramISRConfig.dma.dstChannel = dmaDestChannelPing;  //TODO sync this define EDMA channel to trigger to copy the data out
    //paramISRConfig.cpu.callbackArg = paramSetSem;//TODO check if NULL is required
    errCode = HWA_enableParamSetInterrupt(handle,paramsetIdx,&paramISRConfig);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_enableParamSetInterrupt(PING DMA) returned %d\n",errCode);
        return;
    }

    /***********************/
    /* PONG DUMMY PARAMSET */
    /***********************/
    paramsetIdx++;
    hwaParamCfg[paramsetIdx].triggerMode = HWA_TRIG_MODE_DMA;
    hwaParamCfg[paramsetIdx].dmaTriggerSrc = dmaTriggerSourcePong; //in demo this will be second EDMA Src channel id
    hwaParamCfg[paramsetIdx].accelMode = HWA_ACCELMODE_NONE; //dummy
    errCode = HWA_configParamSet(handle,paramsetIdx,&hwaParamCfg[paramsetIdx],NULL);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configParamSet(%d) returned %d\n",errCode,paramsetIdx);
        return;
    }

    /***********************/
    /* PONG PROCESS PARAMSET */
    /***********************/
    paramsetIdx++;
    hwaParamCfg[paramsetIdx] = hwaParamCfg[pingParamSetIdx];
    hwaParamCfg[paramsetIdx].dest.dstAddr = hwaMemDestPongOffset;
    errCode = HWA_configParamSet(handle,paramsetIdx,&hwaParamCfg[paramsetIdx],NULL);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configParamSet(%d) returned %d\n",errCode,paramsetIdx);
        return;
    }
    /* enable the DMA hookup to this paramset so that data gets copied out */
    paramISRConfig.interruptTypeFlag = HWA_PARAMDONE_INTERRUPT_TYPE_DMA;
    paramISRConfig.dma.dstChannel = dmaDestChannelPong;
    //paramISRConfig.cpu.callbackArg = paramSetSem;//TODO check if NULL is required
    errCode = HWA_enableParamSetInterrupt(handle,paramsetIdx,&paramISRConfig);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_enableParamSetInterrupt(PING DMA) returned %d\n",errCode);
        return;
    }
}


/**
 *  @b Description
 *  @n
 *      Configure HWA for 1D processing.
 *  @param[in]  obj Pointer to data path obj.
 *  @retval none.
 */
void demo_config1D_HWA(Demo_DataPathObj *obj)
{
    int32_t                 errCode;
    HWA_CommonConfig hwaCommonConfig;
    uint8_t          hwaTriggerMode;

    /* Disable the HWA */
    errCode = HWA_enable(obj->hwaHandle,0); // set 1 to enable
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_enable(1) returned %d\n",errCode);
        return;
    }
    if(obj->dataPathMode == DATA_PATH_STANDALONE)
    {
        /* trigger manually and immediately */
        hwaTriggerMode = HWA_TRIG_MODE_SOFTWARE;
    }
    else
    {
        /* trigger done by ADC buffer */
        hwaTriggerMode = HWA_TRIG_MODE_DFE;
    }

    HWAutil_configRangeFFT(obj->hwaHandle,
        MMW_HWA_START_POS_PARAMSETS_1D,
        obj->numAdcSamples,
        obj->numRangeBins,
        obj->numRxAntennas,
        MMW_HWA_WINDOWRAM_1D_OFFSET,
        MMW_HWA_DMA_TRIGGER_SOURCE_1D_PING,
        MMW_HWA_DMA_TRIGGER_SOURCE_1D_PONG,
        MMW_HWA_DMA_DEST_CHANNEL_1D_PING,
        MMW_HWA_DMA_DEST_CHANNEL_1D_PONG,
        ADDR_TRANSLATE_CPU_TO_HWA(MMW_HWA_1D_ADCBUF_INP),
        ADDR_TRANSLATE_CPU_TO_HWA(MMW_HWA_1D_OUT_PING),
        ADDR_TRANSLATE_CPU_TO_HWA(MMW_HWA_1D_OUT_PONG),
        hwaTriggerMode
    );

    /***********************/
    /* HWA COMMON CONFIG   */
    /***********************/
    /* Config Common Registers */
    hwaCommonConfig.configMask = HWA_COMMONCONFIG_MASK_NUMLOOPS |
                                 HWA_COMMONCONFIG_MASK_PARAMSTARTIDX |
                                 HWA_COMMONCONFIG_MASK_PARAMSTOPIDX |
                                 HWA_COMMONCONFIG_MASK_FFT1DENABLE |
                                 HWA_COMMONCONFIG_MASK_INTERFERENCETHRESHOLD;
    hwaCommonConfig.numLoops = obj->numChirpsPerFrame / 2;
    hwaCommonConfig.paramStartIdx = MMW_HWA_START_POS_PARAMSETS_1D;
    hwaCommonConfig.paramStopIdx = MMW_HWA_START_POS_PARAMSETS_1D + HWAUTIL_NUM_PARAM_SETS_1D - 1;
    if(obj->dataPathMode == DATA_PATH_STANDALONE)
    {
        /* HWA will input data from M0 memory*/
        hwaCommonConfig.fftConfig.fft1DEnable = HWA_FEATURE_BIT_DISABLE;
    }
    else
    {
        /* HWA will input data from ADC buffer memory*/
        hwaCommonConfig.fftConfig.fft1DEnable = HWA_FEATURE_BIT_ENABLE;
    }
    hwaCommonConfig.fftConfig.interferenceThreshold = 0xFFFFFF;
    errCode = HWA_configCommon(obj->hwaHandle,&hwaCommonConfig);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configCommon returned %d\n",errCode);
        return;
    }
}


/**
 *  @b Description
 *  @n
 *      EDMA transfer completion call back function as per EDMA API.
 *      Depending on the programmed transfer completion codes,
 *      posts the corresponding done/completion semaphore.
 *      Per current design, a single semaphore could have been used as the
 *      1D, 2D and CFAR stages are sequential, this code gives some flexibility
 *      if some design change in future.
 */
void demo_EDMA_transferCompletionCallbackFxn(uintptr_t arg,
    uint8_t transferCompletionCode)
{
    Demo_DataPathObj *obj = &gMmwMCB.dataPathObj;

    if (transferCompletionCode == DEMO_EDMA_TRANSFER_COMPLETION_CODE_1D_DONE)
      Semaphore_post(obj->EDMA_1Ddone_semHandle);
    else
      demo_printf("Error: EDMA xfr complete returned invalid code: %d\n", transferCompletionCode);
}


/**
 *  @b Description
 *  @n
 *      Call back function for EDMA CC (Channel controller) error as per EDMA API.
 *      Declare fatal error if happens, the output errorInfo can be examined if code
 *      gets trapped here.
 */
void demo_EDMA_errorCallbackFxn(EDMA_Handle handle, EDMA_errorInfo_t *errorInfo)
{
    gMmwMCB.dataPathObj.EDMA_errorInfo = *errorInfo;
    demo_printf("Error: EDMA error callback code: %d\n", *errorInfo);
}


/**
 *  @b Description
 *  @n
 *      Call back function for EDMA transfer controller error as per EDMA API.
 *      Declare fatal error if happens, the output errorInfo can be examined if code
 *      gets trapped here.
 */
void demo_EDMA_transferControllerErrorCallbackFxn(EDMA_Handle handle,
                EDMA_transferControllerErrorInfo_t *errorInfo)
{
    gMmwMCB.dataPathObj.EDMA_transferControllerErrorInfo = *errorInfo;
    demo_printf("Error: EDMA xfr error callback code: %d\n", *errorInfo);
}


static int32_t EDMA_setup_shadow_link (EDMA_Handle handle, uint32_t chId, uint32_t linkChId,
    EDMA_paramSetConfig_t *config, EDMA_transferCompletionCallbackFxn_t transferCompletionCallbackFxn,
    uintptr_t transferCompletionCallbackFxnArg)
{
    EDMA_paramConfig_t paramConfig;
    int32_t errorCode = EDMA_NO_ERROR;

    paramConfig.paramSetConfig = *config; //this will copy the entire param set config
    paramConfig.transferCompletionCallbackFxn = transferCompletionCallbackFxn;
    paramConfig.transferCompletionCallbackFxnArg = (uintptr_t) transferCompletionCallbackFxnArg;
    if ((errorCode = EDMA_configParamSet(handle, linkChId, &paramConfig)) != EDMA_NO_ERROR)
    {
        demo_printf("Error: EDMA_configParamSet() failed with error code = %d\n", errorCode);
        goto exit;
    }

    if ((errorCode = EDMA_linkParamSets(handle, chId, linkChId)) != EDMA_NO_ERROR)
    {
        demo_printf("Error: EDMA_linkParamSets() failed with error code = %d\n", errorCode);
        goto exit;
    }

    if ((errorCode = EDMA_linkParamSets(handle, linkChId, linkChId)) != EDMA_NO_ERROR)
    {
        demo_printf("Error: EDMA_linkParamSets() failed with error code = %d\n", errorCode);
        goto exit;
    }

exit:
    return(errorCode);
}


int32_t EDMAutil_configHwaTranspose(EDMA_Handle handle,
    uint8_t chId, uint16_t linkChId, uint8_t chainChId,
    uint32_t* pSrcAddress, uint32_t * pDestAddress,
    uint8_t numAnt, uint16_t numRangeBins, uint16_t numChirpsPerFrame,
    bool isIntermediateChainingEnabled,
    bool isFinalChainingEnabled,
    bool isTransferCompletionEnabled,
    EDMA_transferCompletionCallbackFxn_t transferCompletionCallbackFxn,
    uintptr_t transferCompletionCallbackFxnArg)
{
    EDMA_channelConfig_t config;
    int32_t errorCode = EDMA_NO_ERROR;

    config.channelId = chId;
    config.channelType = (uint8_t)EDMA3_CHANNEL_TYPE_DMA;
    config.paramId = chId;
    config.eventQueueId = 0;

    config.paramSetConfig.sourceAddress = (uint32_t) pSrcAddress;
    config.paramSetConfig.destinationAddress = (uint32_t) pDestAddress;

    config.paramSetConfig.aCount = numAnt*4;
    config.paramSetConfig.bCount = numRangeBins;
    config.paramSetConfig.cCount = numChirpsPerFrame/2;
    config.paramSetConfig.bCountReload = 0; //config.paramSetConfig.bCount;

    config.paramSetConfig.sourceBindex = numAnt*4;
    config.paramSetConfig.destinationBindex = numChirpsPerFrame*numAnt*4;

    config.paramSetConfig.sourceCindex = 0;
    config.paramSetConfig.destinationCindex = numAnt*4*2;

    config.paramSetConfig.linkAddress = EDMA_NULL_LINK_ADDRESS;
    config.paramSetConfig.transferType = (uint8_t)EDMA3_SYNC_AB;
    config.paramSetConfig.transferCompletionCode = chainChId;
    config.paramSetConfig.sourceAddressingMode = (uint8_t) EDMA3_ADDRESSING_MODE_LINEAR;
    config.paramSetConfig.destinationAddressingMode = (uint8_t) EDMA3_ADDRESSING_MODE_LINEAR;

    /* don't care because of linear addressing modes above */
    config.paramSetConfig.fifoWidth = (uint8_t) EDMA3_FIFO_WIDTH_8BIT;
    config.paramSetConfig.isStaticSet = false;
    config.paramSetConfig.isEarlyCompletion = false;
    config.paramSetConfig.isFinalTransferInterruptEnabled =
        isTransferCompletionEnabled;
    config.paramSetConfig.isIntermediateTransferInterruptEnabled = false;
    config.paramSetConfig.isFinalChainingEnabled =
        isFinalChainingEnabled;
    config.paramSetConfig.isIntermediateChainingEnabled =
        isIntermediateChainingEnabled;
    config.transferCompletionCallbackFxn = transferCompletionCallbackFxn;

    if (transferCompletionCallbackFxn != NULL) {
        config.transferCompletionCallbackFxnArg = (uintptr_t) transferCompletionCallbackFxnArg;
    }

    if ((errorCode = EDMA_configChannel(handle, &config, true)) != EDMA_NO_ERROR)
    {
        demo_printf("Error: EDMA_configChannel() failed with error code = %d\n", errorCode);
        goto exit;
    }

    errorCode = EDMA_setup_shadow_link(handle, chId, linkChId,
        &config.paramSetConfig, config.transferCompletionCallbackFxn, transferCompletionCallbackFxnArg);

exit:
    return(errorCode);
}


int32_t EDMAutil_configHwaOneHotSignature(EDMA_Handle handle,
    uint8_t chId, bool isEventTriggered,
    uint32_t* pSrcAddress, uint32_t * pDestAddress,
    uint16_t aCount, uint16_t bCount, uint16_t cCount,
    uint16_t linkChId)
{
    EDMA_channelConfig_t config;
    int32_t errorCode = EDMA_NO_ERROR;

    config.channelId = chId;
    config.channelType = (uint8_t)EDMA3_CHANNEL_TYPE_DMA;
    config.paramId = chId;
    config.eventQueueId = 0;

    config.paramSetConfig.sourceAddress = (uint32_t) pSrcAddress;
    config.paramSetConfig.destinationAddress = (uint32_t) pDestAddress;

    config.paramSetConfig.aCount = aCount;
    config.paramSetConfig.bCount = bCount;
    config.paramSetConfig.cCount = cCount;
    config.paramSetConfig.bCountReload = config.paramSetConfig.bCount;

    config.paramSetConfig.sourceBindex = 0;
    config.paramSetConfig.destinationBindex = 0;

    config.paramSetConfig.sourceCindex = 0;
    config.paramSetConfig.destinationCindex = 0;

    config.paramSetConfig.linkAddress = EDMA_NULL_LINK_ADDRESS;
    config.paramSetConfig.transferType = (uint8_t)EDMA3_SYNC_A;
    config.paramSetConfig.transferCompletionCode = 0;
    config.paramSetConfig.sourceAddressingMode = (uint8_t) EDMA3_ADDRESSING_MODE_LINEAR;
    config.paramSetConfig.destinationAddressingMode = (uint8_t) EDMA3_ADDRESSING_MODE_LINEAR;

    /* don't care because of linear addressing modes above */
    config.paramSetConfig.fifoWidth = (uint8_t) EDMA3_FIFO_WIDTH_8BIT;

    config.paramSetConfig.isStaticSet = false;
    config.paramSetConfig.isEarlyCompletion = false;
    config.paramSetConfig.isFinalTransferInterruptEnabled = false;
    config.paramSetConfig.isIntermediateTransferInterruptEnabled = false;
    config.paramSetConfig.isFinalChainingEnabled = false;
    config.paramSetConfig.isIntermediateChainingEnabled = false;
    config.transferCompletionCallbackFxn = NULL;

    if ((errorCode = EDMA_configChannel(handle, &config, isEventTriggered)) != EDMA_NO_ERROR)
    {
        demo_printf("Error: EDMA_configChannel() failed with error code = %d\n", errorCode);
        goto exit;
    }

    errorCode = EDMA_setup_shadow_link(handle, chId, linkChId,
        &config.paramSetConfig, config.transferCompletionCallbackFxn, NULL);

exit:
    return(errorCode);
}


/**
 *  @b Description
 *  @n
 *      Configures all 1D processing related EDMA configuration.
 *
 *  @param[in] obj Pointer to data path object
 *  @retval EDMA error code, see EDMA API.
 */
int32_t demo_config1D_EDMA(Demo_DataPathObj *obj)
{
    int32_t errorCode = EDMA_NO_ERROR;
    EDMA_Handle handle = obj->edmaHandle;
    HWA_SrcDMAConfig dmaConfig;

    /* Ping configuration to transfer 1D FFT output from HWA to L3 RAM transposed */
    errorCode = EDMAutil_configHwaTranspose(handle,
        MMW_EDMA_1D_PING_CH_ID,       //chId,
        MMW_EDMA_1D_PING_SHADOW_LINK_CH_ID,  //linkChId,
        MMW_EDMA_1D_PING_CHAIN_CH_ID, //chainChId,
        (uint32_t*)SOC_translateAddress((uint32_t)MMW_HWA_1D_OUT_PING,SOC_TranslateAddr_Dir_TO_EDMA,NULL),   //*pSrcAddress,
        (uint32_t*)SOC_translateAddress((uint32_t)obj->radarCube,SOC_TranslateAddr_Dir_TO_EDMA,NULL),  //*pDestAddress,
        obj->numRxAntennas,           //numAnt,
        obj->numRangeBins,        //numRangeBins,
        obj->numChirpsPerFrame,  //numChirpsPerFrame,
        true,   //isIntermediateChainingEnabled,
        true,   //isFinalChainingEnabled,
        false, //isTransferCompletionEnabled
        NULL,  //transferCompletionCallbackFxn
        NULL);

    if (errorCode != EDMA_NO_ERROR)
    {
        goto exit;
    }

    /* Pong configuration to transfer 1D FFT output from HWA to L3 RAM transposed */
    errorCode = EDMAutil_configHwaTranspose(handle,
        MMW_EDMA_1D_PONG_CH_ID,      //chId,
        MMW_EDMA_1D_PONG_SHADOW_LINK_CH_ID, //linkChId,
        MMW_EDMA_1D_PONG_CHAIN_CH_ID, //SR_DBT_xxx //chainChId,
        (uint32_t*)SOC_translateAddress((uint32_t)MMW_HWA_1D_OUT_PONG,SOC_TranslateAddr_Dir_TO_EDMA,NULL), //*pSrcAddress,
        (uint32_t*) SOC_translateAddress((uint32_t)(obj->radarCube + obj->numRxAntennas),SOC_TranslateAddr_Dir_TO_EDMA,NULL), //*pDestAddress,
         obj->numRxAntennas,          //numAnt,
         obj->numRangeBins,       //numRangeBins,
         obj->numChirpsPerFrame, //numChirpsPerFrame,
         true, //isIntermediateChainingEnabled,
         true, //isFinalChainingEnabled,
         true, //isTransferCompletionEnabled
         demo_EDMA_transferCompletionCallbackFxn, //transferCompletionCallbackFxn
        (uintptr_t)obj);

    if (errorCode != EDMA_NO_ERROR)
    {
        goto exit;
    }

     /**************************************************************************
      *  PROGRAM EDMA2_ACC_CHAN0 (resp. EDMA2_ACC_CHAN1) to communicate completion
         of DMA CHAN EDMA_TPCC0_REQ_HWACC_0 (resp. EDMA_TPCC0_REQ_HWACC_1)
      *************************************************************************/

    HWA_getDMAconfig(obj->hwaHandle, MMW_HWA_DMA_TRIGGER_SOURCE_1D_PING, &dmaConfig);

    errorCode = EDMAutil_configHwaOneHotSignature(handle,
        MMW_EDMA_1D_PING_CHAIN_CH_ID, //chId,
        false, //isEventTriggered
        (uint32_t*)SOC_translateAddress(dmaConfig.srcAddr,SOC_TranslateAddr_Dir_TO_EDMA,NULL), //pSrcAddress
        (uint32_t*)SOC_translateAddress(dmaConfig.destAddr,SOC_TranslateAddr_Dir_TO_EDMA,NULL), //pDestAddress
        dmaConfig.aCnt,
        dmaConfig.bCnt,
        dmaConfig.cCnt,
        MMW_EDMA_1D_PING_ONE_HOT_SHADOW_LINK_CH_ID); //linkChId

    if (errorCode != EDMA_NO_ERROR)
    {
        goto exit;
    }

    HWA_getDMAconfig(obj->hwaHandle, MMW_HWA_DMA_TRIGGER_SOURCE_1D_PONG, &dmaConfig);

    errorCode = EDMAutil_configHwaOneHotSignature(handle,
        MMW_EDMA_1D_PONG_CHAIN_CH_ID, //chId,
        false, //isEventTriggered
        (uint32_t*)SOC_translateAddress(dmaConfig.srcAddr,SOC_TranslateAddr_Dir_TO_EDMA,NULL),  //pSrcAddress
        (uint32_t*)SOC_translateAddress(dmaConfig.destAddr,SOC_TranslateAddr_Dir_TO_EDMA,NULL), //pDestAddress
        dmaConfig.aCnt,
        dmaConfig.bCnt,
        dmaConfig.cCnt,
        MMW_EDMA_1D_PONG_ONE_HOT_SHADOW_LINK_CH_ID); //linkChId

    if (errorCode != EDMA_NO_ERROR)
    {
        goto exit;
    }

exit:
    return(errorCode);
}


/**
 *  @b Description
 *  @n
 *      Trigger HWA for 1D processing.
 *  @param[in]  obj Pointer to data path obj.
 *  @retval none.
 */
void demo_dataPathTrigger1D(Demo_DataPathObj *obj)
{
    int32_t errCode;

    /* Enable the HWA */
    errCode = HWA_enable(obj->hwaHandle, 1); // set 1 to enable
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_enable(1) returned %d\n",errCode);
        return;
    }

    /* trigger the HWA since triggerMode is set to DMA */
    errCode = HWA_setDMA2ACCManualTrig(obj->hwaHandle, MMW_HWA_DMA_TRIGGER_SOURCE_1D_PING);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_setDMA2ACCManualTrig(0) returned %d\n",errCode);
        return;
    }
    errCode = HWA_setDMA2ACCManualTrig(obj->hwaHandle, MMW_HWA_DMA_TRIGGER_SOURCE_1D_PONG);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_setDMA2ACCManualTrig(1) returned %d\n",errCode);
        return;
    }
}


/**
 *  @b Description
 *  @n
 *      Waits for 1D processing to finish. This is a blocking function.
 *
 *  @param[in] obj  Pointer to data path object
 *
 *  @retval
 *      NONE
 */
void demo_dataPathWait1D(Demo_DataPathObj *obj)
{
    Bool       status;

    /**********************************************/
    /* WAIT FOR HWA NUMLOOPS INTERRUPT            */
    /**********************************************/
    /* wait for the all paramSets done interrupt */
    status = Semaphore_pend(obj->HWA_done_semHandle, BIOS_WAIT_FOREVER);
    if (status != TRUE)
    {
      datapath_err |= 0x01;

      demo_printf("Error: Semaphore_pend returned %d\n",status);
    }

    status = Semaphore_pend(obj->EDMA_1Ddone_semHandle, BIOS_WAIT_FOREVER);
    if (status != TRUE)
    {
      datapath_err |= 0x01;
    }
}


/**
 *  @b Description
 *  @n
 *  This function is called at the init time from @ref demo_initTask.
 *  It initializes drivers: ADCBUF, HWA, EDMA, and semaphores used
 *  by  @ref demo_dataPathTask
 *  @retval
 *      Not Applicable.
 */
void demo_dataPathInit(Demo_DataPathObj *obj)
{
    uint8_t edmaNumInstances, inst;
    Semaphore_Params       semParams;
    int32_t errorCode;

    /* Initialize the ADCBUF */
    ADCBuf_init();

    /* Initialize the HWA */
    HWA_init();

    edmaNumInstances = EDMA_getNumInstances();
    for (inst = 0; inst < edmaNumInstances; inst++)
    {
        errorCode = EDMA_init(inst);
        if (errorCode != EDMA_NO_ERROR)
        {
            demo_printf ("Debug: EDMA instance %d initialization returned error %d\n", errorCode);
            return;
        }
        demo_printf ("Debug: EDMA instance %d has been initialized\n", inst);
    }

    memset(&obj->EDMA_errorInfo, 0, sizeof(obj->EDMA_errorInfo));
    memset(&obj->EDMA_transferControllerErrorInfo, 0, sizeof(obj->EDMA_transferControllerErrorInfo));

    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    obj->EDMA_1Ddone_semHandle = Semaphore_create(0, &semParams, NULL);

    /* Enable Done Interrupt */
    Semaphore_Params_init(&semParams);
    semParams.mode = Semaphore_Mode_BINARY;
    obj->HWA_done_semHandle = Semaphore_create(0, &semParams, NULL);
}


void demo_dataPathOpen(Demo_DataPathObj *obj)
{
    int32_t                 errCode;
    EDMA_instanceInfo_t edmaInstanceInfo;
    EDMA_errorConfig_t errorConfig;
    ADCBuf_Params       ADCBufparams;

    /* Open the HWA Instance */
    obj->hwaHandle = HWA_open(0, gMmwMCB.socHandle, &errCode);
    if (obj->hwaHandle == NULL)
    {
        demo_printf("Error: Unable to open the HWA Instance err:%d\n",errCode);
        return;
    }

    demo_printf("Debug: HWA Instance %p has been opened successfully\n", obj->hwaHandle);

    obj->edmaHandle = EDMA_open(0, &errCode, &edmaInstanceInfo);

    if (obj->edmaHandle == NULL)
    {
        demo_printf("Error: Unable to open the EDMA Instance err:%d\n",errCode);
        return;
    }
    demo_printf("Debug: EDMA Instance %p has been opened successfully\n", obj->edmaHandle);

    errorConfig.isConfigAllEventQueues = true;
    errorConfig.isConfigAllTransferControllers = true;
    errorConfig.isEventQueueThresholdingEnabled = true;
    errorConfig.eventQueueThreshold = EDMA_EVENT_QUEUE_THRESHOLD_MAX;
    errorConfig.isEnableAllTransferControllerErrors = true;
    errorConfig.callbackFxn = demo_EDMA_errorCallbackFxn;
    errorConfig.transferControllerCallbackFxn = demo_EDMA_transferControllerErrorCallbackFxn;
    if ((errCode = EDMA_configErrorMonitoring(obj->edmaHandle, &errorConfig)) != EDMA_NO_ERROR)
    {
        demo_printf("Debug: EDMA_configErrorMonitoring() failed with errorCode = %d\n", errCode);
        return;
    }

    /*****************************************************************************
     * Start ADCBUF driver:
     *****************************************************************************/
    /* ADCBUF Params initialize */
    ADCBuf_Params_init(&ADCBufparams);
    ADCBufparams.chirpThreshold = 1;
    ADCBufparams.continousMode  = 0;

    /* Open ADCBUF driver */
    obj->adcbufHandle = ADCBuf_open(0, &ADCBufparams);
    if (obj->adcbufHandle == NULL)
    {
        demo_printf("Error: Unable to open the ADCBUF driver\n");
        return;
    }
    demo_printf("Debug: ADCBUF Instance(0) %p has been opened successfully\n", obj->adcbufHandle);
}


uint32_t fft_winBuf[DEMOCFG_FFT_SIZE/2];
/**
 *  @b Description
 *  @n
 *      Common configuration of data path processing, required to be done only
 *      once. Configures all of EDMA and window RAM of HWA.
 *  @param[in]  obj Pointer to data path obj.
 *  @retval none.
 */
void demo_dataPathConfigCommon(Demo_DataPathObj *obj)
{
    int32_t errCode;
    uint32_t winLen;
    float phi;
    HWA_CommonConfig hwaCommonConfig;

    demo_config1D_EDMA(obj);

    /**********************************************/
    /* Disable HWA and reset to known state       */
    /**********************************************/

    /* Disable the HWA */
    errCode = HWA_enable(obj->hwaHandle,0);
    if (errCode != 0)
    {
        demo_printf("Error: demo_dataPathConfigCommon:HWA_enable(0) returned %d\n",errCode);
        return;
    }

    /* Reset the internal state of the HWA */
    errCode = HWA_reset(obj->hwaHandle);
    if (errCode != 0)
    {
        demo_printf("Error: demo_dataPathConfigCommon:HWA_reset returned %d\n",errCode);
        return;
    }

    /***********************/
    /* CONFIG WINDOW RAM   */
    /***********************/
    /* if windowing is enabled, load the window coefficients in RAM */
    //1D-FFT window

    winLen = obj->numAdcSamples;
    phi = 2 * PI / ((float) winLen - 1);

    demo_genTable(fft_winBuf, winLen/2, phi);

    errCode = HWA_configRam(obj->hwaHandle,
                            HWA_RAM_TYPE_WINDOW_RAM,
                            (uint8_t *)&fft_winBuf[0],
                            sizeof(uint32_t) * winLen, //size in bytes
                            0); //offset in bytes
    if (errCode != 0)
    {
        demo_printf("Error: HWA_configRam returned %d\n",errCode);
        return;
    }

    /**********************************************/
    /* ENABLE NUMLOOPS DONE INTERRUPT FROM HWA */
    /**********************************************/
    errCode = HWA_enableDoneInterrupt(obj->hwaHandle,
        demo_dataPathHwaDoneIsrCallback,
        obj->HWA_done_semHandle);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_enableDoneInterrupt returned %d\n",errCode);
        return;
    }

    /***********************************************/
    /* ENABLE FFT Twiddle coefficient dithering    */
    /***********************************************/
    /* Config Common Registers */
    hwaCommonConfig.configMask = HWA_COMMONCONFIG_MASK_TWIDDITHERENABLE |
                                 HWA_COMMONCONFIG_MASK_LFSRSEED;
    hwaCommonConfig.fftConfig.twidDitherEnable = HWA_FEATURE_BIT_ENABLE;
    hwaCommonConfig.fftConfig.lfsrSeed = 0x1234567; /*Some non-zero value*/

    errCode = HWA_configCommon(obj->hwaHandle,&hwaCommonConfig);
    if (errCode != 0)
    {
        //retCode = HWA_TEST_ERROR;
        demo_printf("Error: HWA_configCommon returned %d\n",errCode);
        return;
    }
}


/**
 *  @b Description
 *  @n
 *      The function is used to configure the data path based on the chirp profile.
 *      After this function is executed, the data path processing will ready to go
 *      when the ADC buffer starts receiving samples corresponding to the chirps.
 *
 *  @retval
 *      Not Applicable.
 */
void demo_dataPathConfig(void)
{
    ADCBuf_dataFormat    dataFormat;
    ADCBuf_RxChanConf    rxChanConf;
    uint32_t             chirpThreshold;
    uint8_t              channel;
    int32_t              retVal = 0;
    uint8_t              numRxAntennas = 0;
    uint8_t              numBytePerSample = 0;
    Demo_ADCBufCfg      *ptrAdcbufCfg;
    Demo_DataPathObj *dataPathObj = &gMmwMCB.dataPathObj;

    ptrAdcbufCfg = &gMmwMCB.adcbufCfg;

    /* Calculate the DMA transfer parameters */
    if (ptrAdcbufCfg->adcFmt == 0)
    {
        /* Complex dataFormat has 4 bytes */
        numBytePerSample =  4;
    }
    else
    {
        /* Real dataFormat has 2 bytes */
        numBytePerSample =  2;
    }

    /* Configure ADC buffer data format */
    dataFormat.adcOutFormat       = ptrAdcbufCfg->adcFmt;
    dataFormat.sampleInterleave   = ptrAdcbufCfg->iqSwapSel;
    dataFormat.channelInterleave  = ptrAdcbufCfg->chInterleave;

    /* Debug Message: */
    demo_printf("Debug: Start ADCBuf driver dataFormat=%d, sampleSwap=%d, interleave=%d, chirpThreshold=%d\n",
                   dataFormat.adcOutFormat, dataFormat.sampleInterleave, dataFormat.channelInterleave,
                   ptrAdcbufCfg->chirpThreshold);

    retVal = ADCBuf_control(dataPathObj->adcbufHandle, ADCBufMMWave_CMD_CONF_DATA_FORMAT,
                           (void *)&dataFormat);
    if (retVal < 0)
    {
        demo_printf ("Error: Unable to configure the data formats\n");
        return;
    }

    memset((void*)&rxChanConf, 0, sizeof(ADCBuf_RxChanConf));

    /* Enable Rx Channels. */
    for (channel = 0; channel < SYS_COMMON_NUM_RX_CHANNEL; channel++)
    {
        if (gMmwMCB.openCfg.chCfg.rxChannelEn & (0x1 << channel))
        {
            /* Populate the receive channel configuration: */
            rxChanConf.channel = channel;
            retVal = ADCBuf_control(dataPathObj->adcbufHandle, ADCBufMMWave_CMD_CHANNEL_ENABLE,
                                   (void *)&rxChanConf);
            if (retVal < 0)
            {
                demo_printf("Error: ADCBuf Control for Channel %d Failed\n");
                return;
            }

            rxChanConf.offset += dataPathObj->numAdcSamples * numBytePerSample;

            /* Track the number of receive channels: */
            numRxAntennas++;
        }
    }

    /* Save number of channels */
    dataPathObj->numRxAntennas = numRxAntennas;

    /* Set the chirp threshold: */
    chirpThreshold = ptrAdcbufCfg->chirpThreshold;
    retVal = ADCBuf_control(dataPathObj->adcbufHandle, ADCBufMMWave_CMD_SET_CHIRP_THRESHHOLD,
                           (void *)&chirpThreshold);
    if (retVal < 0)
    {
        demo_printf("Error: ADCbuf Chirp Threshold Failed with Error[%d]\n", retVal);
        return;
    }

    uint32_t numTxAntAzim = 0;
    uint32_t numTxAntElev = 0;
    dataPathObj->numTxAntennas = 0;
    if (gMmwMCB.openCfg.chCfg.txChannelEn & 0x1)
    {
        numTxAntAzim++;
    }
    if (gMmwMCB.openCfg.chCfg.txChannelEn & 0x4)
    {
        numTxAntAzim++;
    }
    if (gMmwMCB.openCfg.chCfg.txChannelEn & 0x2)
    {
        numTxAntElev++;
    }

    dataPathObj->numTxAntennas = numTxAntAzim + numTxAntElev;

    dataPathObj->numVirtualAntAzim = numTxAntAzim * dataPathObj->numRxAntennas;
    dataPathObj->numVirtualAntElev = numTxAntElev * dataPathObj->numRxAntennas;
    dataPathObj->numVirtualAntennas = dataPathObj->numVirtualAntAzim + dataPathObj->numVirtualAntElev;

    dataPathObj->numAdcSamples = gMmwMCB.profileCfg.numAdcSamples;
    dataPathObj->numRangeBins = demo_pow2roundup(dataPathObj->numAdcSamples);
    dataPathObj->numChirpsPerFrame = (gMmwMCB.ctrlCfg.u.frameCfg.frameCfg.chirpEndIdx -
                                      gMmwMCB.ctrlCfg.u.frameCfg.frameCfg.chirpStartIdx + 1) *
        gMmwMCB.ctrlCfg.u.frameCfg.frameCfg.numLoops;
    dataPathObj->numDopplerBins = dataPathObj->numChirpsPerFrame/dataPathObj->numTxAntennas;

    dataPathObj->rangeResolution = DEMOCFG_SPEED_OF_LIGHT_IN_METERS * gMmwMCB.profileCfg.digOutSampleRate * 1e3 /
                (2 * gMmwMCB.profileCfg.freqSlopeConst * ((3.6*1e3*900)/(1U << 26)) * 1e12 * dataPathObj->numRangeBins);


    demo_memPoolReset(&gMmwL3heap);

    dataPathObj->radarCube = (uint32_t *) demo_memPoolAlloc(&gMmwL3heap, dataPathObj->numRangeBins *
            dataPathObj->numDopplerBins * dataPathObj->numVirtualAntennas * sizeof(uint32_t));

    if (dataPathObj->radarCube == NULL)
    {
        demo_printf("Failed to allocate radar Cube memory from L3 RAM\n");
    }

    dataPathObj->rangeDopplerLogMagMatrix = (uint16_t *) demo_memPoolAlloc(&gMmwL3heap, dataPathObj->numRangeBins *
        dataPathObj->numDopplerBins * sizeof(uint16_t));

    if (dataPathObj->rangeDopplerLogMagMatrix == NULL)
    {
        demo_printf("Failed to allocate Range Dopppler Log Matrix memory from L3 RAM\n");
    }

    #ifdef RANGE_DC_CALIBRATION
    if (obj->dcRangeSigMean == NULL)
    {
        obj->dcRangeSigMean = (cmplx32ImRe_t *) Memory_alloc (NULL,
             DC_RANGE_SIGNATURE_COMP_MAX_BIN_SIZE * SOC_MAX_NUM_TX_ANTENNAS * SOC_MAX_NUM_RX_ANTENNAS * sizeof(cmplx32ImRe_t),
              (uint8_t) MMWDEMO_MEMORY_ALLOC_DOUBLE_WORD_ALIGN,
              NULL);
    }
    #endif

    demo_dataPathConfigCommon(dataPathObj);
    demo_config1D_HWA(dataPathObj);
    demo_dataPathTrigger1D(dataPathObj);

    return;
}


#ifdef CZ_BOARD
//This is a capture of CZ board full cal.
uint32_t cz_chunk1[56] = {
0x800017FE, 0x800017FE, 0x0000000F, 0x000017FE,
0x001D001D, 0x00000002, 0x0000001F, 0x2C181010,
0x00000000, 0x00000000, 0x00000000, 0xFFFE0114,
0x00001611, 0x00003401, 0x0112A880, 0x00000000,
0x00001C01, 0x01C9C380, 0x00000000, 0x00062C01,
0x0146F95B, 0x00000192, 0x00120901, 0x017D7840,
0x000001C3, 0x0157001D, 0x011A0136, 0x00ED0101,
0x558E38E4, 0x58E38E38, 0x00000000, 0x00000000,
0x00000000, 0x100A0476, 0x00000000, 0x00000000,
0x00000000, 0xF0FA03F9, 0x00000000, 0x00000000,
0x00000000, 0xFF3E0412, 0x00000000, 0x00000000,
0x00000000, 0xFE3003BD, 0x1912001D, 0x3D302321,
0x0003FFCC, 0x004D0043, 0x007E0071, 0x0008FFD6,
0x004D0044, 0x007D0070, 0x00000000, 0x00000000};

uint32_t cz_chunk2[56] = {
0x00000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x000F0001, 0xFFF30140,
0x000B0142, 0x00020140, 0xFFFF0140, 0x00050142,
0xFFF80140, 0x000F0102, 0x000B0140, 0x00190144,
0x001D0140, 0xFFFF0142, 0x00180142, 0xFFEF0142,
0x000C0140, 0x001A0102, 0xFFFF0144, 0xFFE90141,
0x00000140, 0x0012014A, 0x00100102, 0xFFFD0106,
0xFFF1010F, 0xFFE70104, 0x00100108, 0xFFF8010A,
0xFFFE0148, 0xFFFC0140, 0xFFF5015C, 0x001D010A,
0x00140104, 0x00010142, 0xFFFF0106, 0xFFF90103,
0xFFE90108, 0xFFE6010A, 0x000C0154, 0x001B0108,
0x00070150, 0x001D0140, 0xFFFF0142, 0xFFFF0102,
0x001B0106, 0xFFFB014C, 0xFFFB0140, 0x001B010A};

uint32_t cz_chunk3[56] = {
0x001D0140, 0xFFE50104, 0x00000154, 0x000D0151,
0x00040102, 0xFFF50102, 0xFFF90144, 0x00160140,
0xFFF9014E, 0xFFE30146, 0x0017014A, 0x001D0144,
0x000006CF, 0x00000000, 0x00000000, 0x06D40000,
0x00000000, 0x00000000, 0x00000000, 0x00000699,
0x00000000, 0x00000000, 0x06D50000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x06BC0000,
0x06CC06C9, 0x069606DE, 0x06D706AB, 0x024A06E8,
0x08040F06, 0x5C40480A, 0x0642040A, 0x540A0803,
0x42405008, 0x404C0602, 0x5404400A, 0x44020251,
0x4A464E40, 0x00000044, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00000000,
0x00000000, 0x00000000, 0x00000000, 0x00000000};
#endif


//This is a copy of MMWave_internalLinkSync().
static void demo_internalLinkSync(MMWave_MCB *ptrMMWaveMCB,
                                  uint32_t    flag)
{
    int32_t completed = 0;

    /* Loop around till the condition is met: */
    while (completed == 0)
    {
        /* Is the flag set? */
        if (ptrMMWaveMCB->linkStatus & flag)
        {
            /* YES: Flag has been set. */
            completed = 1;
        }
    }
    return;
}


//This is a copy of MMWave_openLink. It is needed to control what calibration is
//performed, and whether to write stored calibration data back to the BSS.
#define MMWAVE_RF_OPERATIONAL               1

int32_t demo_openLink(MMWave_MCB        *ptrMMWaveMCB,
                      MMWave_OpenCfg    *openCfg,
                      uint32_t           calMask,
                      rlCalibrationData_t  *calData,
                      int32_t           *errCode)
{
    int32_t                     retVal = -1;
    rlRfCalMonFreqLimitConf_t   freqLimit;
    rlRfInitCalConf_t           rfInitCalib;
    rlRfCalMonTimeUntConf_t     timeCfg;
    rlRfLdoBypassCfg_t          ldoCfg;

    /* Initialize the configurations: */
    memset ((void *)&freqLimit, 0, sizeof(rlRfCalMonFreqLimitConf_t));
    memset ((void *)&rfInitCalib, 0, sizeof(rlRfInitCalConf_t));
    memset ((void *)&timeCfg, 0, sizeof(rlRfCalMonTimeUntConf_t));
    memset ((void *)&ldoCfg, 0, sizeof(rlRfLdoBypassCfg_t));


    /* Link is not operational: */
    ptrMMWaveMCB->linkStatus = 0U;

    /* Setup the RF Calibration Time unit
     * - Our periodicity is set to 1 Frame  */
    timeCfg.calibMonTimeUnit = 1U;
    retVal = rlRfSetCalMonTimeUnitConfig(RL_DEVICE_MAP_INTERNAL_BSS, &timeCfg);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: calibration init failed */
        *errCode =  MMWAVE_ECALPERIOD;
        goto exit;
    }

    //Parameter for LDO Bypass. This is expressly linked to the
    //PMIC type and mode that it is functioning. If using the standard LP87524B PMIC keep
    //this parameter a 0. If using the installed LP87524J PMIC in PFM Mode (default configuration for high efficiency)
    //then you should also keep this a zero. If you are using in forced PWM mode then and only then
    //should this value be a 1. Setting this value to a 1 in any other circumstance will result in both device
    //and board damage.

    ldoCfg.ldoBypassEnable = 0;// 0 for No bypass (1.3 V, default), 1 for Yes Bypass (1.0 V).
#ifndef PMIC_MODE_PFM

#ifdef PMIC_TYPE_J
    ldoCfg.ldoBypassEnable = 1;// 0 for No bypass (1.3 V, default), 1 for Yes Bypass (1.0 V).
#endif
#endif

    retVal = rlRfSetLdoBypassConfig(RL_DEVICE_MAP_INTERNAL_BSS, &ldoCfg);


    /* Setup the RF Calibration Frequency limit as specified by the user configuration: */
    freqLimit.freqLimitLow  = openCfg->freqLimitLow;
    freqLimit.freqLimitHigh = openCfg->freqLimitHigh;
    retVal = rlRfSetCalMonFreqLimitConfig(RL_DEVICE_MAP_INTERNAL_BSS, &freqLimit);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: Frequency Limit Calibration  */
        *errCode =  MMWAVE_ECALCFG;
        goto exit;
    }

    rfInitCalib.calibEnMask = calMask;

    retVal = rlRfInitCalibConfig(RL_DEVICE_MAP_INTERNAL_BSS, &rfInitCalib);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: calibration init failed */
        *errCode =  MMWAVE_ECALINIT;
        goto exit;
    }

    /* Set the channel configuration: */
    retVal = rlSetChannelConfig (RL_DEVICE_MAP_INTERNAL_BSS, &openCfg->chCfg);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: Set the channel configuration failed */
        *errCode = MMWAVE_ECHCFG;
        goto exit;
    }

    /* Set the ADC Output configuration: */
    retVal = rlSetAdcOutConfig (RL_DEVICE_MAP_INTERNAL_BSS, &openCfg->adcOutCfg);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: Set the ADC configuration failed */
        *errCode = MMWAVE_EADCCFG;
        goto exit;
    }

    /* Set the low power mode configuration */
    retVal = rlSetLowPowerModeConfig(RL_DEVICE_MAP_INTERNAL_BSS, &openCfg->lowPowerMode);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: Set the Low power configuration failed */
        *errCode = MMWAVE_EPOWERCFG;
        goto exit;
    }

    /****************************************************************************************
     * Asynchronous event configuration:
     ****************************************************************************************/
    retVal = MMWave_deviceCfgAsyncEvent (ptrMMWaveMCB, errCode);
    if (retVal < 0)
    {
        /* Error: Asynchronous Event configuration failed; error code is already setup */
        goto exit;
    }

    //Write the calibration table to BSS if needed.
    if (calData != NULL)
    {
        retVal = rlRfCalibDataRestore(RL_DEVICE_MAP_INTERNAL_BSS, calData);
        if (retVal != RL_RET_CODE_OK)
        {
            /* Error: Calibration table restore failed */
            *errCode = MMWAVE_ERFINIT;
            goto exit;
        }

        //wait for the sync
        demo_internalLinkSync(ptrMMWaveMCB, MMWAVE_RF_OPERATIONAL);

        //clear the flag so it will wait again for rlRfInit
        ptrMMWaveMCB->linkStatus &= ~(MMWAVE_RF_OPERATIONAL);
    }

    /* Init RF: */
    retVal = rlRfInit(RL_DEVICE_MAP_INTERNAL_BSS);
    if (retVal != RL_RET_CODE_OK)
    {
        /* Error: RF Initialization failed */
        *errCode =  MMWAVE_ERFINIT;
        goto exit;
    }

    /***************************************************************************
     * SYNCHRONIZATION: We need to loop around till the BSS has performed the
     * RF Initialization.
     ***************************************************************************/
    demo_internalLinkSync(ptrMMWaveMCB, MMWAVE_RF_OPERATIONAL);

    /* Setup the return value: */
    retVal = 0;

exit:
    return retVal;
}


//This function takes one of the pre-defined chirp configurations and loads
//it into the control structures with which to initialize the mmWave module.
int32_t demo_staticConfig(void)
{
  int32_t  errCode;
  uint32_t flashAddr;
  MMWave_ProfileHandle   profileHandle;
  MMWave_MCB            *ptrMMWaveMCB;

#ifdef POWER_TEST
  //Power testing: set GPIO0 low:
  gpio_write(0, 0, 0);
  cptime[3] = Cycleprofiler_getTimeStamp();
#endif

  rlAdcOutCfg_t       *adcOutCfg  = &gMmwMCB.openCfg.adcOutCfg;
  Demo_ADCBufCfg      *adcbufCfg  = &gMmwMCB.adcbufCfg;
  rlProfileCfg_t      *profileCfg = &gMmwMCB.profileCfg;
  rlLowPowerModeCfg_t *lowPwrMode = &gMmwMCB.openCfg.lowPowerMode;
  rlFrameCfg_t        *frameCfg   = &gMmwMCB.ctrlCfg.u.frameCfg.frameCfg;
  rlChanCfg_t         *chCfg      = &gMmwMCB.openCfg.chCfg;
  rlChirpCfg_t        *chirpCfg   = &gMmwMCB.chirpCfg;
  MMWave_CalibrationCfg *calCfg   = &gMmwMCB.calCfg;
  Demo_DataPathObj *dataPathObj   = &gMmwMCB.dataPathObj;

  /* Initialize the control configuration from the static config: */
  memset ((void*)&gMmwMCB.ctrlCfg, 0, sizeof(MMWave_CtrlCfg));

  /* Populate the channel configuration: */
#ifndef CZ_BOARD
  chCfg->rxChannelEn            = 0x01; //Rx enable mask
#else
  //FOR CAL TEMP TESTING (w/waveguide)
  chCfg->rxChannelEn            = 0x08; //Rx enable mask
#endif
  chCfg->txChannelEn            = 0x01; //Tx enable mask
  chCfg->cascading              = 0;

  /* Populate the ADC output configuration: */
  adcOutCfg->fmt.b2AdcBits   = 2; //ADC_BITS_16
  adcOutCfg->fmt.b2AdcOutFmt = 1; //complex 1x

  /* Populate the low power configuration: */
  lowPwrMode->reserved = 0;
  lowPwrMode->lpAdcMode = 0;

  /* Setup the calibration frequency: */
  gMmwMCB.openCfg.freqLimitLow  = DEMO_CAL_FREQ_LOW;
  gMmwMCB.openCfg.freqLimitHigh = DEMO_CAL_FREQ_HIGH;

  ptrMMWaveMCB = (MMWave_MCB *)gMmwMCB.ctrlHandle;

#ifdef POWER_TEST
  cptime[4] = Cycleprofiler_getTimeStamp();
#endif

#ifndef RUN_FULL_CAL
  //Read the init calibration table from flash.
  flashAddr = QSPIFlash_getExtFlashAddr(qspiflashHandle) +
              DEMO_CAL_FLASH_OFFSET;

  QSPIFlash_singleRead(qspiflashHandle, flashAddr, DEMO_CAL_FLASH_LEN,
                      (uint8_t *)init_cal_data);
#endif

#ifdef CZ_BOARD
  //For CZ board testing, load full cal data from C array:
  init_cal_data[0].numOfChunk = 3;
  init_cal_data[0].chunkId    = 0;
  memcpy(&init_cal_data[0].calData, cz_chunk1, 56*4);
  init_cal_data[1].numOfChunk = 3;
  init_cal_data[1].chunkId    = 1;
  memcpy(&init_cal_data[1].calData, cz_chunk2, 56*4);
  init_cal_data[2].numOfChunk = 3;
  init_cal_data[2].chunkId    = 2;
  memcpy(&init_cal_data[2].calData, cz_chunk3, 56*4);
#endif

  /* Open the mmWave module: */

#ifdef RUN_FULL_CAL
  demo_openLink(ptrMMWaveMCB, &gMmwMCB.openCfg, 0x17F0, NULL, &errCode);
#else
  demo_openLink(ptrMMWaveMCB, &gMmwMCB.openCfg, 0x0010, init_cal_data, &errCode);
#endif
  if (errCode < 0)
  {
    demo_printf ("Error: mmWave Open failed (code %d)\n", errCode);
    return -1;
  }

  /* The module has been opened successfully: */
  ptrMMWaveMCB->status = ptrMMWaveMCB->status | MMWAVE_STATUS_OPENED;

  /* Populate the profile configuration: */
  profileCfg->profileId             = 0;
  profileCfg->pfVcoSelect           = 0;
  profileCfg->pfCalLutUpdate        = 0;
  profileCfg->startFreqConst        = DEMOCFG_START_FREQCONST; //77 * 18641351; //(2^26) / 3.6;
  profileCfg->idleTimeConst         = DEMOCFG_IDLE_TIME;
  profileCfg->adcStartTimeConst     = DEMOCFG_ADC_START_TIME;
  profileCfg->rampEndTime           = DEMOCFG_RAMP_END_TIME;
#ifndef CZ_BOARD
  profileCfg->txOutPowerBackoffCode = DEMOCFG_TxBackoff;
#else
  profileCfg->txOutPowerBackoffCode = 8;
#endif
  profileCfg->txPhaseShifter        = 0;
  profileCfg->freqSlopeConst        = DEMOCFG_FREQ_SLOPE;
  profileCfg->txStartTime           = 100;
  profileCfg->numAdcSamples         = DEMOCFG_NUM_ADC_SAMPLES;
  profileCfg->digOutSampleRate      = DEMOCFG_SAMPLE_RATE;
  profileCfg->hpfCornerFreq1        = 0;
  profileCfg->hpfCornerFreq2        = 0;
#ifndef CZ_BOARD
  profileCfg->rxGain                = DEMOCFG_RxLNAGAIN;
#else
  profileCfg->rxGain                = 24;
#endif

  /* Populate the chirp configuration: */
  chirpCfg->chirpStartIdx   = 0;
  chirpCfg->chirpEndIdx     = 0;
  chirpCfg->profileId       = 0;
  chirpCfg->startFreqVar    = 0;
  chirpCfg->freqSlopeVar    = 0;
  chirpCfg->idleTimeVar     = 0;
  chirpCfg->adcStartTimeVar = 0;
  chirpCfg->txEnable        = 1;

  /* Populate the frame configuration: */
  frameCfg->chirpStartIdx       = 0;
  frameCfg->chirpEndIdx         = 0;
  frameCfg->numLoops            = DEMOCFG_NUMLOOPS;
  frameCfg->numAdcSamples       = DEMOCFG_NUM_ADC_SAMPLES;
  frameCfg->triggerSelect       = 1; //Software API based triggering
  frameCfg->frameTriggerDelay   = 0;
  #ifndef MULTI_FRAME
  frameCfg->numFrames           = 1;
  frameCfg->framePeriodicity    = 2000000;
  #else
  frameCfg->numFrames           = 100;
  frameCfg->framePeriodicity    = 20000;
  #endif

  /* Populate Adcbuf configuration: */
  adcbufCfg->adcFmt             = 0; //0=complex
  adcbufCfg->iqSwapSel          = 1; //1=Q in LSB, I in MSB
  adcbufCfg->chInterleave       = 1; //not interleaved
  adcbufCfg->chirpThreshold     = 1;

  dataPathObj->numRxAntennas   = 1;   //computed from Rx chan enables in _adcbufCfg
  dataPathObj->noiseEnergy     = 29552; //not used
  dataPathObj->numTxAntennas   = 1;  //computed from Tx channel enables
  dataPathObj->numAdcSamples   = DEMOCFG_NUM_ADC_SAMPLES; //copied from _profileCfg
  dataPathObj->numRangeBins    = DEMOCFG_FFT_SIZE; //recomputed
  dataPathObj->numChirpsPerFrame = 1; //computed from frameCfg chirp indexes and numLoops
  dataPathObj->rangeResolution = DEMOCFG_RANGE_RES; //computed; dist res in meters
  dataPathObj->dataPathMode    = DATA_PATH_WITH_ADCBUF;

  /* Populate the runtime calibration configuration: */
  #ifdef RUN_ONESHOT_CAL
  calCfg->u.chirpCalibrationCfg.enableCalibration    = true;  //enable one-shot
  calCfg->u.chirpCalibrationCfg.enablePeriodicity    = false; //disable periodic runtime
  calCfg->u.chirpCalibrationCfg.periodicTimeInFrames = 0;
  #else
  calCfg->u.chirpCalibrationCfg.enableCalibration    = false;
  calCfg->u.chirpCalibrationCfg.enablePeriodicity    = false;
  calCfg->u.chirpCalibrationCfg.periodicTimeInFrames = 0;
  #endif
  calCfg->dfeDataOutputMode = MMWave_DFEDataOutputMode_FRAME;

  //Create profile 0.
  profileHandle = MMWave_addProfile(gMmwMCB.ctrlHandle, profileCfg, &errCode);
  gMmwMCB.ctrlCfg.u.frameCfg.profileHandle[0] = profileHandle;

  //Add the chirp definition to the profile
  gMmwMCB.chirpHandle = MMWave_addChirp(profileHandle, chirpCfg, &errCode);

  gMmwMCB.ctrlCfg.dfeDataOutputMode = MMWave_DFEDataOutputMode_FRAME;

  /* Configure the mmWave module: */
  if (MMWave_config (gMmwMCB.ctrlHandle, &gMmwMCB.ctrlCfg, &errCode) < 0)
  {
    demo_printf("Error: mmWave Config failed (code %d)\n", errCode);
    return -1;
  }

#ifdef POWER_TEST
  cptime[5] = Cycleprofiler_getTimeStamp();
#endif

  /* Setup the data path following MMWave_config: */
  demo_dataPathConfig();
  return 0;
}


int32_t demo_startSensor(void)
{
  int32_t errCode;

#ifdef POWER_TEST
  //Power testing: set GPIO0 high:
  gpio_write(0, 0, 1);
  cptime[6] = Cycleprofiler_getTimeStamp();
#endif

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  //This will set GPIO 0 (header pin J6.15) low as a flag to
  //the MSP432 that mmwave and chirp configuration has completed.
#ifndef POWER_TEST
  gpio_write(0, 0, 0);
#endif
  #endif

  /* Start the mmWave module: The configuration has been applied successfully. */
  if (MMWave_start (gMmwMCB.ctrlHandle, &gMmwMCB.calCfg, &errCode) < 0)
  {
    /* Error: Unable to start the mmWave control */
    demo_printf ("Error: mmWave Control Start failed (code %d)\n", errCode);
    return -1;
  }

  return 0;
}


#ifdef RANGE_DC_CALIBRATION
/**
 *  @b Description
 *  @n
 *    Compensation of DC range antenna signature.  This function will examine
 *    the first and last several bins (configurable) over the chirps in the frame,
 *    and will remove the average power from these bins.
 *
 *  @retval
 *      Not Applicable.
 */
void demo_dcRangeSignatureCompensation(Demo_DataPathObj *obj)
{
    uint32_t antIdx, rngBinIdx;
    uint32_t ind;
    int32_t dcRangeSigMeanSize;
    uint32_t doppIdx;
    cmplx16ImRe_t *fftOut1D = (cmplx16ImRe_t *) obj->radarCube;

    dcRangeSigMeanSize = obj->numVirtualAntennas * (obj->calibDcRangeSigCfg.positiveBinIdx - obj->calibDcRangeSigCfg.negativeBinIdx + 1);
    if (obj->dcRangeSigCalibCntr == 0)
    {
        memset(obj->dcRangeSigMean, 0, dcRangeSigMeanSize * sizeof(cmplx32ImRe_t));
    }

    /* Calibration */
    if (obj->dcRangeSigCalibCntr < obj->calibDcRangeSigCfg.numAvgChirps)
    {
        for (doppIdx = 0; doppIdx < obj->numDopplerBins; doppIdx++)
        {
            /* Accumulate */
            ind = 0;
            for (rngBinIdx = 0; rngBinIdx <= obj->calibDcRangeSigCfg.positiveBinIdx; rngBinIdx++)
            {
                for (antIdx  = 0; antIdx < obj->numVirtualAntennas; antIdx++)
                {
                    obj->dcRangeSigMean[ind].real +=
                          fftOut1D[ rngBinIdx * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].real;
                    obj->dcRangeSigMean[ind].imag +=
                          fftOut1D[ rngBinIdx * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].imag;
                    ind++;
                }
            }
            for (rngBinIdx = 0; rngBinIdx < -obj->calibDcRangeSigCfg.negativeBinIdx; rngBinIdx++)
            {
                for (antIdx  = 0; antIdx < obj->numVirtualAntennas; antIdx++)
                {
                    obj->dcRangeSigMean[ind].real +=
                          fftOut1D[ (obj->numRangeBins + obj->calibDcRangeSigCfg.negativeBinIdx + rngBinIdx) * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].real;
                    obj->dcRangeSigMean[ind].imag +=
                          fftOut1D[ (obj->numRangeBins + obj->calibDcRangeSigCfg.negativeBinIdx + rngBinIdx) * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].imag;
                    ind++;
                }
            }
            obj->dcRangeSigCalibCntr++;
        }

        if (obj->dcRangeSigCalibCntr == obj->calibDcRangeSigCfg.numAvgChirps)
        {
            /* Divide */
            for (ind  = 0; ind < (obj->numVirtualAntennas * dcRangeSigMeanSize); ind++)
            {
                obj->dcRangeSigMean[ind].real = obj->dcRangeSigMean[ind].real >> obj->log2NumAvgChirps;
                obj->dcRangeSigMean[ind].imag = obj->dcRangeSigMean[ind].imag >> obj->log2NumAvgChirps;
            }
        }
    }
    else
    {
        /* fftOut1D -= dcRangeSigMean */
        for (doppIdx = 0; doppIdx < obj->numDopplerBins; doppIdx++)
        {
            ind = 0;
            for (rngBinIdx = 0; rngBinIdx <= obj->calibDcRangeSigCfg.positiveBinIdx; rngBinIdx++)
            {
                for (antIdx  = 0; antIdx < obj->numVirtualAntennas; antIdx++)
                {
                    fftOut1D[ rngBinIdx * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].real -= obj->dcRangeSigMean[ind].real;
                    fftOut1D[ rngBinIdx * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].imag -= obj->dcRangeSigMean[ind].imag;
                    ind++;
                }
            }
            for (rngBinIdx = 0; rngBinIdx < -obj->calibDcRangeSigCfg.negativeBinIdx; rngBinIdx++)
            {
                for (antIdx  = 0; antIdx < obj->numVirtualAntennas; antIdx++)
                {
                    fftOut1D[ (obj->numRangeBins + obj->calibDcRangeSigCfg.negativeBinIdx + rngBinIdx) * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].real -= obj->dcRangeSigMean[ind].real;
                    fftOut1D[ (obj->numRangeBins + obj->calibDcRangeSigCfg.negativeBinIdx + rngBinIdx) * (obj->numVirtualAntennas * obj->numDopplerBins) + doppIdx * obj->numVirtualAntennas + antIdx].imag -= obj->dcRangeSigMean[ind].imag;
                    ind++;
                }
            }
        }
    }
}
#endif


/**
 *  @b Description
 *  @n
 *      Registered event function to mmwave which is invoked when an event from the
 *      BSS is received.
 *
 *  @param[in]  msgId
 *      Message Identifier
 *  @param[in]  sbId
 *      Subblock identifier
 *  @param[in]  sbLen
 *      Length of the subblock
 *  @param[in]  payload
 *      Pointer to the payload buffer
 *
 *  @retval
 *      Always return 0
 */
int32_t bss_debugAssert = 0;
int32_t bss_EsmAssert = 0;

int32_t demo_eventCallbackFxn(uint16_t msgId, uint16_t sbId, uint16_t sbLen, uint8_t *payload)
{
    uint16_t asyncSB = RL_GET_SBID_FROM_UNIQ_SBID(sbId);

    /* Process the received message: */
    switch (msgId)
    {
        case RL_RF_ASYNC_EVENT_MSG:
        {
            /* Received Asychronous Message: */
            switch (asyncSB)
            {
                case RL_RF_AE_CPUFAULT_SB:
                {
                    bss_debugAssert = 1;
                    break;
                }
                case RL_RF_AE_ESMFAULT_SB:
                {
                    bss_EsmAssert = 1;
                    break;
                }
                case RL_RF_AE_INITCALIBSTATUS_SB:
                {
                    rlRfInitComplete_t*  ptrRFInitCompleteMessage;

                    /* Get the RF-Init completion message: */
                    ptrRFInitCompleteMessage = (rlRfInitComplete_t*)payload;
                    gMmwMCB.dataPathObj.calibrationStatus = ptrRFInitCompleteMessage->calibStatus & 0x1FFF;
                    break;
                }
#if 0
                case RL_RF_AE_FRAME_TRIGGER_RDY_SB:
                {
                    gMmwMCB.stats.frameTriggerReady++;
                    break;
                }
                case RL_RF_AE_MON_TIMING_FAIL_REPORT_SB:
                {
                    gMmwMCB.stats.failedTimingReports++;
                    break;
                }
                case RL_RF_AE_RUN_TIME_CALIB_REPORT_SB:
                {
                    gMmwMCB.stats.calibrationReports++;
                    break;
                }
                case RL_RF_AE_FRAME_END_SB:
                {
                    /*Received Frame Stop async event from BSS.
                      No action required.*/
                    break;
                }
#endif
                default:
                {
                    demo_printf("Error: Asynchronous Event SB Id %d not handled\n", asyncSB);
                    break;
                }
            }
            break;
        }
        default:
        {
            demo_printf("Error: Asynchronous message %d is NOT handled\n", msgId);
            break;
        }
    }
    return 0;
}


/**
 *  @b Description
 *  @n
 *      The task is used to provide an execution context for the mmWave
 *      control task
 *
 *  @retval
 *      Not Applicable.
 */
void demo_mmWaveCtrlTask(UArg arg0, UArg arg1)
{
    int32_t errCode;

    while (1)
    {
        /* Execute the mmWave control module: */
        if (MMWave_execute (gMmwMCB.ctrlHandle, &errCode) < 0)
            demo_printf ("Error: mmWave control execution failed [Error code %d]\n", errCode);
    }
}


/**
 *  @b Description
 *  @n
 *      This task is used to run full calibration and save the resulting
 *      calibration table to flash for subsequent runs.
 *
 *  @retval
 *      Not Applicable.
 */
void demo_calibrationTask(UArg arg0, UArg arg1)
{
  uint32_t   flashAddr;
  int32_t    errCode;
  uint32_t  *calptr;
  SPI_Transaction   transaction;
  MMWave_MCB       *ptrMMWaveMCB;
  rlChanCfg_t      *chCfg      = &gMmwMCB.openCfg.chCfg;
  rlAdcOutCfg_t    *adcOutCfg  = &gMmwMCB.openCfg.adcOutCfg;
  rlLowPowerModeCfg_t *lowPwrMode = &gMmwMCB.openCfg.lowPowerMode;

  gpio_led_on();  //turn the user LED on

  /* Get the pointer to the control module */
  ptrMMWaveMCB = (MMWave_MCB*)gMmwMCB.ctrlHandle;

  /* Populate the channel configuration: */
#ifndef CZ_BOARD
  chCfg->rxChannelEn            = 0x01; //Rx enable mask
#else
  //FOR CALIBRATION TEMP TESTING (CZ board + waveguide)
  chCfg->rxChannelEn            = 0x08; //Rx enable mask
#endif
  chCfg->txChannelEn            = 0x01; //Tx enable mask
  chCfg->cascading              = 0;

  /* Populate the ADC output configuration: */
  adcOutCfg->fmt.b2AdcBits   = 2; //ADC_BITS_16
  adcOutCfg->fmt.b2AdcOutFmt = 1; //complex 1x

  /* Populate the low power configuration: */
  lowPwrMode->reserved        = 0;
  lowPwrMode->lpAdcMode        = 0;

  gMmwMCB.openCfg.freqLimitLow  = DEMO_CAL_FREQ_LOW;
  gMmwMCB.openCfg.freqLimitHigh = DEMO_CAL_FREQ_HIGH;

 /* Copy over the configuration: */
  memcpy ((void *)&ptrMMWaveMCB->openCfg, &gMmwMCB.openCfg, sizeof(MMWave_OpenCfg));

  //Run full calibration
  errCode = 0;
  demo_openLink(ptrMMWaveMCB, &gMmwMCB.openCfg, 0x17F0, NULL, &errCode);

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
    //set GPIO_0 low when cal finishes
    gpio_write(0, 0, 0);
  #endif

  if (errCode == 0)
  {
    //Grab the calibration table (chunks) from the BSS.
    errCode = rlRfCalibDataStore(RL_DEVICE_MAP_INTERNAL_BSS, (rlCalibrationData_t *)init_cal_data);

    if (errCode == 0)
    {
      //Do some sanity checks before writing it to flash
      calptr = (uint32_t *)init_cal_data;
      if ((calptr[  0] != 0x00000003) ||
          (calptr[ 57] != 0x00010003) ||
          (calptr[114] != 0x00020003))
      {
        errCode = -1;
        cal_data.pass = -3; //invalid cal headers
      }
      else if ((calptr[1] & 0x1fff) != DEMO_GOOD_CAL_RESULT)
      {
        errCode = -1;
        cal_data.pass = -4; //not all calibrations passed
      }
    }
    else
      cal_data.pass = -2; //rlRfCalibDataStore failed

    if (errCode == 0) //so far, so good
    {
      #ifndef RUN_FULL_CAL
      flashAddr = QSPIFlash_getExtFlashAddr(qspiflashHandle) +
                  DEMO_CAL_FLASH_OFFSET;

      /* Erase the 4KB sector to be written. */
      QSPIFlash_sectorErase(qspiflashHandle, flashAddr);

      QSPIFlash_singleWrite(qspiflashHandle, flashAddr, DEMO_CAL_FLASH_LEN,
                           (uint8_t *)init_cal_data);
      #endif

      cal_data.pass = DEMO_MAGIC_CAL_PASS;
      cal_data.numChunks = RL_MAX_CALIB_DATA_CHUNK;
      cal_data.taskMask = calptr[1] & 0x0000ffff;
    }
  }
  else
  {
    errCode = -1;
    cal_data.pass = -1; //MMWave_initRF failed
  }

  if (errCode != 0)
  {
    cal_data.numChunks = 0;
    cal_data.taskMask = calptr[1] & 0x0000ffff;
  }
  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  else
  {
    //set GPIO_0 high to indicate successful calibration
    gpio_write(0, 0, 1);
  }
  #endif

  /* Configure the SPI Data Transfer */
  transaction.count = sizeof(cal_data) / 2;
  transaction.txBuf = &cal_data;
  transaction.rxBuf = NULL;
  transaction.slaveIndex = 0;

  /* Start Data Transfer */
  if (SPI_transfer(spiHandle, &transaction) != true)
  {
    demo_printf("Debug: SPI_transfer write failed\n");
  }

  #ifdef POWER_TEST
    //Power testing - set high again if cal is successful
    if ((cal_data.pass == DEMO_MAGIC_CAL_PASS) &&
        (cal_data.numChunks == RL_MAX_CALIB_DATA_CHUNK) &&
        (cal_data.taskMask == 0x17fe))
      gpio_write(0, 0, 1);
  #endif

  //Now wait for the MSP432 to power down the IWR1443.
  while (1)
  { };
}


#define START_IDX                5
//#define CAL_TESTING

uint32_t chirp_result[DEMOCFG_NUM_ADC_SAMPLES];

//The FFT output bins will be interleaved (by chirp), by the frameCfg parameter
//.numLoops. The corresponding bins from each chirp will be added together to
//improve the SNR.  Also, the first few bins will be ignored due to the radar
//phenomena of having large values there.
void demo_findTopTanges(LSDemo_ChirpData *results)
{
  uint32_t  idx;
  uint32_t  idx2;
  uint32_t  idx3;
  uint32_t  chirps;
  int32_t   temp;
  int16_t  *fft_out;
  uint32_t *mag;
  uint32_t  max_power[DEMO_NUM_RESULTS];
  uint32_t  max_bin[DEMO_NUM_RESULTS];
#ifdef CAL_TESTING
  uint32_t  temp_bin[DEMO_NUM_RESULTS];
  uint64_t  sum;
#endif

  mag     = &chirp_result[0];
  chirps  = gMmwMCB.ctrlCfg.u.frameCfg.frameCfg.numLoops;
  idx3    = chirps * 2 * START_IDX;
  fft_out = (int16_t *) gMmwL3;
  fft_out+= idx3;

  for (idx = START_IDX; idx < DEMOCFG_NUM_ADC_SAMPLES; idx ++)
  {
    mag[idx] = 0;

    for (idx2 = 0; idx2 < chirps * 2; idx2 ++)
    {
      //Sum the squares of the magnitudes (of I and Q) and sum the chirps.
      //(don't need to do square roots since we'll only looking for max)
      temp = *fft_out++ >> 1;
      mag[idx] += (temp * temp);
    }
  }

  //Now locate the top N, in order
  for (idx = 0; idx < DEMO_NUM_RESULTS; idx ++)
  {
    max_power[idx] = 0;

    for (idx2 = START_IDX; idx2 < DEMOCFG_NUM_ADC_SAMPLES; idx2 ++)
    {
      if (mag[idx2] > max_power[idx])
      {
        max_power[idx] = mag[idx2];
        max_bin[idx] = idx2;
      }
    }

#ifdef CAL_TESTING
  temp_bin[idx] = mag[max_bin[idx]];
#endif
    mag[max_bin[idx]] = 0;
  }

#ifdef CAL_TESTING
  //Repair the freq bin array then compute average power
  for (idx = 0; idx < DEMO_NUM_RESULTS; idx ++)
    mag[max_bin[idx]] = temp_bin[idx];

  sum = 0;

  for (idx = START_IDX; idx < DEMOCFG_NUM_ADC_SAMPLES; idx ++)
    sum += mag[idx];
#endif

  //Now compute the distances of the top N.
  for (idx = 0; idx < DEMO_NUM_RESULTS; idx ++)
  {
    results->distance[idx]  = (float)((max_bin[idx]) * DEMOCFG_RANGE_RES);
    results->power_sqr[idx] = max_power[idx];
  }

#ifdef CAL_TESTING
  //Replace the last power value with the averaged chirp power
  results->power_sqr[DEMO_NUM_RESULTS-1] = sum / (DEMOCFG_NUM_ADC_SAMPLES-START_IDX);
#endif
}


/**
 *  @b Description
 *  @n
 *      The task is used for data path processing and to transmit the
 *      detected objects to the MSP432 via the SPI port.
 *
 *  @retval
 *      Not Applicable.
 */

void demo_dataPathTask(UArg arg0, UArg arg1)
{
  uint32_t idx;
  int32_t  err;

  SPI_Transaction transaction;
  Demo_DataPathObj *dataPathObj = &gMmwMCB.dataPathObj;

  datapath_err = 0;

  //Configure the static (built-in) chirp parameters and run calibration.
  err = demo_staticConfig();
  if (err != 0)
    err = DEMO_MEAS_ERR_CONFIG;  //error in the configuration

  if (gMmwMCB.dataPathObj.calibrationStatus != DEMO_GOOD_CAL_RESULT)
    err = DEMO_MEAS_ERR_CAL;     //not all calibration tasks passed

  //Communicate with the BSS to start the chirp.
  if (err == 0)
  {
    err = demo_startSensor();
    if (err != 0)
      err = DEMO_MEAS_ERR_START;  //error in starting mmWave control
  }

  #ifdef POWER_TEST
    cptime[7] = Cycleprofiler_getTimeStamp();
  #endif

  if (err == 0)
  {
    #ifndef MULTI_FRAME
    //Perform the Range FFT once, no frame loop
    demo_dataPathWait1D(dataPathObj);

    //Wait for the HWA to finish the range FFT
    while (gMmwMCB.dataPathObj.hwaDoneIsrCounter == 0)
    {


    }

    #else //loop for the configured number of frames
    for (idx = 0; idx < gMmwMCB.ctrlCfg.u.fullControlCfg.u.chirpModeCfg.frameCfg.numFrames; idx ++)
    {
      //Perform the Range FFT once, no frame loop
      demo_dataPathWait1D(dataPathObj);

      //Wait for the HWA to finish the range FFT
      while (gMmwMCB.dataPathObj.hwaDoneIsrCounter == 0)
      {
      }

      /* Prepare for next frame */
      demo_config1D_HWA(dataPathObj);
      demo_dataPathTrigger1D(dataPathObj);
    }
    #endif

    #ifdef REPORT_IR14_TIME_MEASUREMENTS
    #ifdef POWER_TEST
      //Power testing - set low
      gpio_write(0, 0, 0);
      cptime[8] = Cycleprofiler_getTimeStamp();

      //This will set GPIO 0 (header pin J6.15) high again as a flag to
      //the MSP432 that chirp processing has completed.
    #else
      gpio_write(0, 0, 1);
    #endif
    #endif

    //Get the chirp results and send it out over SPI
    if (datapath_err == 0)
    {
      #ifdef RANGE_DC_CALIBRATION

      if (dataPathObj->calibDcRangeSigCfg.enabled)
      {
        if (dataPathObj->calibDcRangeSigCfg.numAvgChirps < dataPathObj->numDopplerBins)
        {
            dataPathObj->calibDcRangeSigCfg.enabled = 0;
            dataPathObj->dcRangeForcedDisableCntr++;
        }
        demo_dcRangeSignatureCompensation(dataPathObj);
      }
      #endif

      demo_findTopTanges(&chirp_data);
    }
    else //1D FFT did not return
    {
      err = DEMO_MEAS_ERR_FFT;

      for (idx = 0; idx < DEMO_NUM_RESULTS; idx ++)
      {
        chirp_data.distance[idx] = 0;
        chirp_data.power_sqr[idx] = 0;
      }
    }
  }

  #ifdef REPORT_IR14_TIME_MEASUREMENTS
  #ifndef POWER_TEST
  if (err != 0)
  {
    //If an early error occurred, fake out the GPIO communication
    if (err == DEMO_MEAS_ERR_CONFIG)
    {
      for (idx = 0; idx < 1000; idx ++) //just burn time
        gpio_write(0, 0, 0);
    }
    if (err != DEMO_MEAS_ERR_FFT)
    {
      for (idx = 0; idx < 1000; idx ++) //just burn time
        gpio_write(0, 0, 1);
    }
  }
  #endif
  #endif

  chirp_data.err = err;
  chirp_data.pad = 0;

  /* Configure the SPI Data Transfer */
  transaction.count = sizeof(chirp_data);
  transaction.txBuf = &chirp_data;
  transaction.rxBuf = NULL;
  transaction.slaveIndex = 0;

  /* Start Data Transfer */
  if (SPI_transfer(spiHandle, &transaction) != true)
  {
    demo_printf("Debug: SPI_transfer write failed\n");
  }

  //Power testing: set GPIO0 high
  #ifdef POWER_TEST
    gpio_write(0, 0, 1);
    cptime[9] = Cycleprofiler_getTimeStamp();
  #endif

  //Now wait for the MSP432 to power down the IWR1443.
  while (1)
  { };
}
