/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
/*******************************************************************************
 * Add description here.
 *
 *
 * This file contains GPIO functions for the MSP432 side demo executable.
 *
 *******************************************************************************/
/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>

#include <ti/drivers/GPIO.h>

//msp432 driverlib
#include "gpio.h"

/* Demo Includes */
#include "msp_gpio.h"



//This function sets IWR1443's GPIO_1 to indicate what to run:
//  1 = calibration
//  0 = measurement
void gpio_set_gpio1(uint32_t value)
{
  if (value)
    GPIO_setOutputHighOnPin(GPIO_PORT_P5, GPIO_PIN7);
  else
    GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN7);
}


int32_t gpio_wait_on_ir14xx_gpio0(uint32_t pend_value)
{
  uint32_t timeout = 0;
  uint32_t value = pend_value + 1;

  GPIO_setAsInputPin(GPIO_PORT_P4, GPIO_PIN6);

  while (value != pend_value)
  {
    value = GPIO_getInputPinValue(GPIO_PORT_P4, GPIO_PIN6);
    timeout ++;
    if (timeout > 500000) return(-1);
  }

  return(0);
}


//This function pulls the IR1443's PMIC_EN pin low, which removes power
//from the IR1443. Resister R102 must be soldered onto the EVM for this
//to function properly.
void gpio_enter_ir1443_reset(void)
{
  GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN7);

  #ifdef PULL_IWR1443_RESET
  GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN6);
  #endif
}


//This function sets the IR1443's PMIC_EN pin high, which allows power
//to the IR1443. Resister R102 must be soldered onto the EVM for this
//to function properly.
void gpio_release_ir1443_reset(void)
{
  GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN7);

  #ifdef PULL_IWR1443_RESET
  GPIO_setOutputHighOnPin(GPIO_PORT_P6, GPIO_PIN6);
  #endif
}

//This function allows the user to relase the nRST line from being held
//low by MSP432.
void gpio_nRST_set_float(void)
{
    GPIO_setAsInputPin(GPIO_PORT_P5, GPIO_PIN6);
}

//This function allows the user to pull the nRST line low by the MSP432
void gpio_nRST_pull_low(void)
{
    GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN6);
    GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_PIN6);
}


//AG: Added,

int32_t gpio_read_PGOOD(void)
{
    GPIO_setAsInputPin(GPIO_PORT_P4, GPIO_PIN5);

    uint32_t value1 = 0;

    if(GPIO_getInputPinValue(GPIO_PORT_P4, GPIO_PIN5))
    {
        value1 = 1;
    }

    return(value1);

}

















void gpio_init(void)
{
  GPIO_init();

  //Currently, GPIO_PIN1..3 are initialized in the UART code
  //Currently, GPIO_PIN5..7 are initialized in the SPI code

  GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN6);
  GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN6);

  #ifdef PULL_IWR1443_RESET
  //If this code is enabled, it will be used to pull the IWR1443 reset pin
  //high and low along with PMIC_EN. This is a more correct boot sequence,
  //but it requires two hardware modifications: 1) The RST pin on the MSP432
  //must be removed, and 2) IWR1443 J5 pins 9 and 10 must be wired together.
  GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_PIN6);
  GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN6);
  #endif

  /* Configure GPIO 4.6 for timing IR1443 boot time (IWR1443 GPIO 0) */
  GPIO_setAsInputPin(GPIO_PORT_P4, GPIO_PIN6);

  /* Configure GPIO 4.7 for IR1443 power control (IR1443 PMIC_EN) */
  GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN7);
  GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN7);

  /* Configure GPIO 5.7 for Program mode (IWR1443 GPIO 0) */
  GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_PIN7);
  GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN7);
}
