/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.
*
* Texas Instruments Incorporated grants a world-wide, royalty-free,
* non-exclusive license under copyrights and patents it now or hereafter
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent
* license, such license is granted  solely to the extent that any such patent is necessary
* to Utilize the software alone.  The patent license shall not apply to any combinations which
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the
* above copyright notice and the disclaimer and (if applicable) source code license limitations below)
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/
/*******************************************************************************
 * Add description here.
 *
 *
 * This file contains SPI functions for the MSP432 side demo executable.
 *
 *******************************************************************************/
/* Standard Includes */
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

/* DriverLib Includes */
#include <ti/drivers/SPI.h>

#include "Board.h"

/* Demo Includes */
#include "demo_types.h"
#include "msp_spi.h"
#include "msp_gpio.h"
#include "demo_common.h"

#define  BIT_SHIFT

/* Globals */

SPI_Handle      spiHandle;
SPI_Transaction spiTransaction;

//uint8_t spiRxBuffer[MSP_SPI_MSG_LENGTH+2];
uint8_t spiRxBuffer[640];
uint8_t spiTxBuffer[MSP_SPI_MSG_LENGTH+2];




//This function performs byte swapping on a buffer. The buffer length
//should be a multiple of 2.
void spi_swap_data(uint8_t *buffer, uint32_t length)
{
  uint32_t idx;
  uint8_t  temp;
//This code is a temporary fix to overcome a right-shift-by-1 problem
//we are currently seeing with the mmwave SDK.
//#ifdef BIT_SHIFT
  /*
#ifndef REPORT_IR14_TIME_MEASUREMENTS
  uint8_t first_msb;
  uint8_t curr_msb;

  first_msb = (buffer[0] & 0x80) >> 7;
  buffer[0] <<= 1;

  for (idx = 1; idx < length; idx++)
  {
    curr_msb = (buffer[idx] & 0x80) >> 7;
    buffer[idx] <<= 1;
    buffer[idx-1] |= curr_msb;
  }

  buffer[length-1] |= first_msb;
#endif
*/
  for (idx = 0; idx < length; idx+=2)
  {
    temp = buffer[idx];
    buffer[idx] = buffer[idx+1];
    buffer[idx+1] = temp;
  }
}


//This function will block until a transfer arrives on SPI. Once it does,
//the data will be byte swapped before returning.
void *spi_get_transfer(uint32_t size)
{
  uint8_t *ret_ptr = spiRxBuffer;

//  gpio_flag_spi_ready(GPIO_ON);

  if (!SPI_transfer(spiHandle, &spiTransaction))
    ret_ptr = NULL;
  else
    /* Byte swap the contents of the rx buffer */
    spi_swap_data(spiRxBuffer, size);

//  gpio_flag_spi_ready(GPIO_OFF);

  return(ret_ptr);
}


void spi_init(void)
{
  SPI_Params spiParams;

  SPI_init();

#ifndef MSP_SPI_MASTER

    /* Initialize SPI handle in slave mode */
    SPI_Params_init(&spiParams);
    spiParams.mode = SPI_SLAVE;
    spiParams.frameFormat =  SPI_POL0_PHA0;
//    spiParams.frameFormat =  SPI_POL1_PHA1;
//    spiParams.transferMode = SPI_MODE_CALLBACK;
//    spiParams.transferCallbackFxn = spiTransferCompleteCallback;
    spiParams.transferMode = SPI_MODE_BLOCKING;
    spiParams.transferTimeout = SPI_WAIT_FOREVER;
    spiParams.bitRate  = 1000000;
//    spiParams.dataSize = 8;
    spiParams.dataSize = 16;

    spiHandle = SPI_open(Board_SPI0, &spiParams);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }

    /* Initialize slave SPI transaction structure */
//    spiTransaction.count = MSP_SPI_MSG_LENGTH;
spiTransaction.count = MSP_SPI_MSG_LENGTH / 2;
    spiTransaction.txBuf = NULL;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

    /* Wait for master to open spi */
//    pthread_barrier_wait(&barrier);
//  ***SEND GPIO SIGNAL HERE TO INDICATE IT IS READY RECEIVE DATA***
//    GPIO_write(Board_GPIO_P4_6, GPIO_ON);
//    Chip Select callback
//    GPIO_setCallback(Board_GPIO_P3_0, spiCSCallbackFxn);
//    // Enable interrupts
//    GPIO_enableInt(Board_GPIO_P3_0);


#else //running MSP432 as SPI Master

    bool transferOK;

    /* Initialize SPI handle as default master */
    spiHandle = SPI_open(Board_SPI0, NULL);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }

//    pthread_barrier_wait(&barrier);

    /* Initialize master SPI transaction structure */
    spiTransaction.count = MSP_SPI_MSG_LENGTH;
    spiTransaction.txBuf = (void *)spiTxBuffer;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

//move the following code
    /* Initiate SPI transfer */
    transferOK = SPI_transfer(spiHandle, &spiTransaction);

    if (transferOK)
    {
        /* Print contents of master receive buffer */
        demo_printf("Master: %s\n", spiRxBuffer);
    }
    else {
        demo_printf("Unsuccessful master SPI transfer");
    }

#endif
}




void spi_init_cal(void)
{
  SPI_Params spiParams;



  SPI_init();

#ifndef MSP_SPI_MASTER

  //AG:
      SPI_close(spiHandle);

    /* Initialize SPI handle in slave mode */
    SPI_Params_init(&spiParams);
    spiParams.mode = SPI_SLAVE;
    spiParams.frameFormat =  SPI_POL0_PHA0;
//    spiParams.frameFormat =  SPI_POL1_PHA1;
//    spiParams.transferMode = SPI_MODE_CALLBACK;
//    spiParams.transferCallbackFxn = spiTransferCompleteCallback;
    spiParams.transferMode = SPI_MODE_BLOCKING;
    spiParams.transferTimeout = SPI_WAIT_FOREVER;
    spiParams.bitRate  = 1000000;
//    spiParams.dataSize = 8;
    spiParams.dataSize = 16;

    spiHandle = SPI_open(Board_SPI0, &spiParams);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }


    /* Initialize slave SPI transaction structure */
//AG: uncommented:
    //spiTransaction.count = MSP_SPI_MSG_LENGTH;
//AG: Commented:
spiTransaction.count = MSP_SPI_MSG_LENGTH / 2;
    spiTransaction.txBuf = NULL;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

    /* Wait for master to open spi */
//    pthread_barrier_wait(&barrier);
//  ***SEND GPIO SIGNAL HERE TO INDICATE IT IS READY RECEIVE DATA***
//    GPIO_write(Board_GPIO_P4_6, GPIO_ON);
//    Chip Select callback
//    GPIO_setCallback(Board_GPIO_P3_0, spiCSCallbackFxn);
//    // Enable interrupts
//    GPIO_enableInt(Board_GPIO_P3_0);


#else //running MSP432 as SPI Master

    bool transferOK;

    /* Initialize SPI handle as default master */
    spiHandle = SPI_open(Board_SPI0, NULL);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }

//    pthread_barrier_wait(&barrier);

    /* Initialize master SPI transaction structure */
    spiTransaction.count = MSP_SPI_MSG_LENGTH;
    spiTransaction.txBuf = (void *)spiTxBuffer;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

//move the following code
    /* Initiate SPI transfer */
    transferOK = SPI_transfer(spiHandle, &spiTransaction);

    if (transferOK)
    {
        /* Print contents of master receive buffer */
        demo_printf("Master: %s\n", spiRxBuffer);
    }
    else {
        demo_printf("Unsuccessful master SPI transfer");
    }

#endif
}



void spi_init_again(void)
{
  SPI_Params spiParams;

  SPI_init();

#ifndef MSP_SPI_MASTER


  //AG:
     SPI_close(spiHandle);
    /* Initialize SPI handle in slave mode */
    SPI_Params_init(&spiParams);
    spiParams.mode = SPI_SLAVE;
    spiParams.frameFormat =  SPI_POL0_PHA0;
//    spiParams.frameFormat =  SPI_POL1_PHA1;
//    spiParams.transferMode = SPI_MODE_CALLBACK;
//    spiParams.transferCallbackFxn = spiTransferCompleteCallback;
    spiParams.transferMode = SPI_MODE_BLOCKING;
    spiParams.transferTimeout = SPI_WAIT_FOREVER;
    spiParams.bitRate  = 1000000;
//    spiParams.dataSize = 8;
    spiParams.dataSize = 16;

    spiHandle = SPI_open(Board_SPI0, &spiParams);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }


    /* Initialize slave SPI transaction structure */
//AG: uncommented:
    spiTransaction.count = MSP_SPI_MSG_LENGTH;
//AG: Commented:
//spiTransaction.count = MSP_SPI_MSG_LENGTH / 2;
    spiTransaction.txBuf = NULL;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

    /* Wait for master to open spi */
//    pthread_barrier_wait(&barrier);
//  ***SEND GPIO SIGNAL HERE TO INDICATE IT IS READY RECEIVE DATA***
//    GPIO_write(Board_GPIO_P4_6, GPIO_ON);
//    Chip Select callback
//    GPIO_setCallback(Board_GPIO_P3_0, spiCSCallbackFxn);
//    // Enable interrupts
//    GPIO_enableInt(Board_GPIO_P3_0);


#else //running MSP432 as SPI Master

    bool transferOK;

    /* Initialize SPI handle as default master */
    spiHandle = SPI_open(Board_SPI0, NULL);
    if (spiHandle == NULL) {
        demo_printf("Error initializing SPI\n");
        while (1);
    }
    else {
        demo_printf("SPI initialized\n");
    }

//    pthread_barrier_wait(&barrier);

    /* Initialize master SPI transaction structure */
    spiTransaction.count = MSP_SPI_MSG_LENGTH;
    spiTransaction.txBuf = (void *)spiTxBuffer;
    spiTransaction.rxBuf = (void *)spiRxBuffer;

//move the following code
    /* Initiate SPI transfer */
    transferOK = SPI_transfer(spiHandle, &spiTransaction);

    if (transferOK)
    {
        /* Print contents of master receive buffer */
        demo_printf("Master: %s\n", spiRxBuffer);
    }
    else {
        demo_printf("Unsuccessful master SPI transfer");
    }

#endif
}
