/*
* Copyright (c) 2017 Texas Instruments Incorporated
*
* All rights reserved not granted herein.
* Limited License.  
*
* Texas Instruments Incorporated grants a world-wide, royalty-free, 
* non-exclusive license under copyrights and patents it now or hereafter 
* owns or controls to make, have made, use, import, offer to sell and sell ("Utilize")
* this software subject to the terms herein.  With respect to the foregoing patent 
* license, such license is granted  solely to the extent that any such patent is necessary 
* to Utilize the software alone.  The patent license shall not apply to any combinations which 
* include this software, other than combinations with devices manufactured by or for TI ("TI Devices").  
* No hardware patent is licensed hereunder.
*
* Redistributions must preserve existing copyright notices and reproduce this license (including the 
* above copyright notice and the disclaimer and (if applicable) source code license limitations below) 
* in the documentation and/or other materials provided with the distribution
*
* Redistribution and use in binary form, without modification, are permitted provided that the following
* conditions are met:
*
*             * No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any 
*               software provided in binary form.
*             * any redistribution and use are licensed by TI for use only with TI Devices.
*             * Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
*
* If software source code is provided to you, modification and redistribution of the source code are permitted 
* provided that the following conditions are met:
*
*   * any redistribution and use of the source code, including any resulting derivative works, are licensed by 
*     TI for use only with TI Devices.
*   * any redistribution and use of any object code compiled from the source code and any resulting derivative 
*     works, are licensed by TI for use only with TI Devices.
*
* Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or 
* promote products derived from this software without specific prior written permission.
*
* DISCLAIMER.
*
* THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
* IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*/
/*******************************************************************************
 *
 * This file contains Timer32 functions for the MSP432 side demo executable.
 *
 *******************************************************************************/

/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>

#include <ti/drivers/Power.h>
#include <ti/drivers/power/PowerMSP432.h>

/* Demo Includes */
#include "Board.h"
#include "demo_types.h"
#include "msp_timer.h"
#include "msp_uart.h"

extern MSP_Demo_Parms demo_parms;


/* Globals */
Timer_Handle      timer_handle;
volatile uint32_t timer_ticks = 0;
volatile uint32_t timer_event = 0;


/* Timer callback function */
void timerCallback(Timer_Handle myHandle)
{
  timer_ticks++;
  timer_event = 1;
  Power_setConstraint(PowerMSP432_DISALLOW_SLEEP);
}


void timer_start(void)
{
  Timer_start(timer_handle);
}


void timer_stop(void)
{
  Timer_stop(timer_handle);
}


#ifdef REPORT_IR14_TIME_MEASUREMENTS
//Setup Timer32_2 to run if we are going to collect timings for IR1443
void timer_start_timer32(void)
{
  uint32_t  *reg;

  reg = (uint32_t *)T32LOAD2;
 *reg = TIMER32_LOAD_VAL;

  reg = (uint32_t *)T32CTRL2;
 *reg = 0x82; //start the timer free running, wrapping, no interrupts
}


//Grab the current value of Timer32_2
uint32_t timer_get_timer32(void)
{
  uint32_t  *reg;

  reg = (uint32_t *)T32VALUE2;
  return(*reg);
}
#endif


void timer_init(void)
{
  Timer_Params params;

  Timer_init();

  /* Setting up the timer in continuous callback mode that calls the callback
   * functions every 1000000us (1s).
   */
  Timer_Params_init(&params);
  params.period = demo_parms.us_per_wakeup;
  params.periodUnits = TIMER_PERIOD_US;
  params.timerMode = TIMER_CONTINUOUS_CB;
  params.timerCallback = timerCallback;

  timer_handle = Timer_open(Board_Timer0, &params);

  if (timer_handle == NULL)
  {
    /* Failed to initialized Timer */
    demo_printf("ERROR: Timer failed to open\n");
    while(1);
  }

  timer_ticks = 0;
  timer_event = 0;

#ifdef REPORT_IR14_TIME_MEASUREMENTS
  timer_start_timer32();
#endif
}
